"""
   Copyright 2006-2008 SpringSource (http://springsource.com), All Rights Reserved

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.       
"""
import logging
import threading
import time
import Pyro4

from socket import getaddrinfo, gethostbyname

pyro_threads = {}
serviceList = {}
logger = logging.getLogger("springpython.remoting.pyro.Pyro4DaemonHolder")

def resolve(host, port):
    canonhost = gethostbyname(host)
    canonport = getaddrinfo(host, port)[0][4][1]
    
    return canonhost, canonport

def register(pyro_obj, service_name, host, port):
    """
    Register the Pyro4 object and its service name with the daemon.
    Also add the service to a dictionary of objects. This allows the
    PyroDaemonHolder to intelligently know when to start and stop the
    daemon thread.
    """
    logger.debug("Registering %s at %s:%s with the Pyro4 server" % (service_name, host, port))

    host, port = resolve(host, port)
    
    serviceList[(service_name, host, port)] = pyro_obj

    if (host, port) not in pyro_threads:
    
        logger.debug("Pyro4 thread needs to be started at %s:%s" % (host, port))
      
        pyro_threads[(host, port)] = _Pyro4Thread(host, port)
        pyro_threads[(host, port)].start()
        
    if not hasattr(pyro_obj, "_pyroId"):
        uri = pyro_threads[(host, port)].pyro_daemon.register(pyro_obj, service_name)

def deregister(service_name, host, port):
    """
    Deregister the named service by removing it from the list of
    managed services and also disconnect from the daemon.
    """
    logger.debug("Deregistering %s at %s:%s with the Pyro4 server" % (service_name, host, port))

    host, port = resolve(host, port)

    if (host, port) in pyro_threads:    
        pyro_threads[(host, port)].pyro_daemon.unregister(serviceList[(service_name, host, port)])
        del(serviceList[(service_name, host, port)])

        def get_address((service_name, host, port)):
            return (host, port)

        if len([True for x in serviceList.keys() if get_address(x) == (host, port)]) == 0:
            shutdown(host, port)

def shutdown(daemon_host, daemon_port):
    """This provides a hook so an application can deliberately shutdown a
    daemon thread."""
    logger.debug("Shutting down Pyro4 daemon at %s:%s" % (daemon_host, daemon_port))

    daemon_host, daemon_port = resolve(daemon_host, daemon_port)

    try:
        pyro_threads[(daemon_host, daemon_port)].shutdown()
        time.sleep(1.0)
        del(pyro_threads[(daemon_host, daemon_port)])
    except Exception, e:
        logger.debug("Failed to shutdown %s:%s => %s" % (daemon_host, daemon_port, e))

class _Pyro4Thread(threading.Thread):
    """
    This is a thread that runs the Pyro4 daemon. It is instantiated automatically
    from within Pyro4ServiceExporter.
    """
    
    def __init__(self, host, port):
        """
        When this class is created, it also created a Pyro4 core daemon to manage.
        """
        threading.Thread.__init__(self)
        self.host = host
        self.port = port
        self.logger = logging.getLogger("springpython.remoting.pyro.Pyro4DaemonHolder._Pyro4Thread")

        self.logger.debug("Creating Pyro4 daemon")
        self.pyro_daemon = Pyro4.Daemon(host=host, port=port)
    
    def run(self):
        """
        When this thread starts up, it initializes the Pyro4 server and then puts the
        daemon into listen mode so it can process remote requests.
        """
        self.logger.debug("Starting up Pyro4 server thread for %s:%s" % (self.host, self.port))
        self.pyro_daemon.requestLoop()

    def shutdown(self):
        """
        This is a hook in order to signal the thread that its time to shutdown
        the Pyro4 daemon.
        """
        self.logger.debug("Signaling shutdown of Pyro4 server thread for %s:%s" % (self.host, self.port))
        class ShutdownThread(threading.Thread):
            def __init__(self, pyro_daemon):
                threading.Thread.__init__(self)
                self.pyro_daemon = pyro_daemon
                self.logger = logging.getLogger("springpython.remoting.pyro.Pyro4DaemonHolder.ShutdownThread")
            def run(self):
                self.logger.debug("Sending shutdown signal...")
                self.pyro_daemon.shutdown()

        ShutdownThread(self.pyro_daemon).start()





"""
   Copyright 2006-2008 SpringSource (http://springsource.com), All Rights Reserved

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.       
"""

import atexit
import logging
from traceback import format_exc

from springpython.container import ObjectContainer

class ApplicationContext(ObjectContainer):
    """
    ApplicationContext IS a ObjectContainer. It also has the ability to define the lifecycle of
    objects.
    """
    def __init__(self, config = None):
        super(ApplicationContext, self).__init__(config)
        
        atexit.register(self.shutdown_hook)
        
        self.logger = logging.getLogger("springpython.context.ApplicationContext")
        self.classnames_to_avoid = set(["PyroProxyFactory", "ProxyFactoryObject", "Pyro4ProxyFactory", "Pyro4FactoryObject"])
         
        for object_def in self.object_defs.values():
            self._apply(object_def)
            
        for configuration in self.configs:
            self._apply(configuration)

        for object_def in self.object_defs.values():
            if not object_def.lazy_init and object_def.id not in self.objects:
                self.logger.debug("Eagerly fetching %s" % object_def.id)
                self.get_object(object_def.id, ignore_abstract=True)

        post_processors = [object for object in self.objects.values() if isinstance(object, ObjectPostProcessor)]

        for obj_name, obj in self.objects.iteritems():
            if not isinstance(obj, ObjectPostProcessor):
                for post_processor in post_processors:
                    self.objects[obj_name] = post_processor.post_process_before_initialization(obj, obj_name)


        for object in self.objects.values():
            self._apply(object)

        for obj_name, obj in self.objects.iteritems():
            if not isinstance(obj, ObjectPostProcessor):
                for post_processor in post_processors:
                    self.objects[obj_name] = post_processor.post_process_after_initialization(obj, obj_name)
            
    def _apply(self, obj):
        if not (obj.__class__.__name__ in self.classnames_to_avoid): 
            if hasattr(obj, "after_properties_set"):
                obj.after_properties_set()
            #if hasattr(obj, "post_process_after_initialization"):
            #    obj.post_process_after_initialization(self)
            if hasattr(obj, "set_app_context"):
                obj.set_app_context(self)
                
    def get_objects_by_type(self, type_, include_type=True):
        """ Returns all objects which are instances of a given type.
        If include_type is False then only instances of the type's subclasses
        will be returned.
        """
        result = {}
        for obj_name, obj in self.objects.iteritems():
            if isinstance(obj, type_):
                if include_type == False and type(obj) is type_:
                    continue
                result[obj_name] = obj
                
        return result
                
    def shutdown_hook(self):
        self.logger.debug("Invoking the destroy_method on registered objects")
        
        for obj_name, obj in self.objects.iteritems():
            if isinstance(obj, DisposableObject):
                try:
                    if hasattr(obj, "destroy_method"):
                        destroy_method_name = getattr(obj, "destroy_method")
                    else:
                        destroy_method_name = "destroy"
                        
                    destroy_method = getattr(obj, destroy_method_name)
                    
                except Exception, e:
                    self.logger.error("Could not destroy object '%s', exception '%s'" % (obj_name, format_exc()))
                    
                else:
                    if callable(destroy_method):
                        try:
                            self.logger.debug("About to destroy object '%s'" % obj_name)
                            destroy_method()
                            self.logger.debug("Successfully destroyed object '%s'" % obj_name)
                        except Exception, e:
                            self.logger.error("Could not destroy object '%s', exception '%s'" % (obj_name, format_exc()))
                    else:
                        self.logger.error("Could not destroy object '%s', " \
                            "the 'destroy_method' attribute it defines is not callable, " \
                            "its type is '%r', value is '%r'" % (obj_name, type(destroy_method), destroy_method))
                        
        self.logger.debug("Successfully invoked the destroy_method on registered objects")
            
class InitializingObject(object):
    """This allows definition of a method which is invoked by the container after an object has had all properties set."""
    def after_properties_set(self):
        pass

class ObjectPostProcessor(object):
    def post_process_before_initialization(self, obj, obj_name):
        return obj
    def post_process_after_initialization(self, obj, obj_name):
        return obj

class ApplicationContextAware(object):
    def __init__(self):
        self.app_context = None
        
    def set_app_context(self, app_context):
        self.app_context = app_context

class ObjectNameAutoProxyCreator(ApplicationContextAware, ObjectPostProcessor):
    """
    This object will iterate over a list of objects, and automatically apply
    a list of advisors to every callable method. This is useful when default advice
    needs to be applied widely with minimal configuration.
    """
    def __init__(self, objectNames = [], interceptorNames = []):
        super(ObjectNameAutoProxyCreator, self).__init__()
        self.objectNames = objectNames
        self.interceptorNames = interceptorNames

class DisposableObject(object):
    """ This allows definition of a method which is invoked when the 
    container's shutting down to release the resources held by an object.
    """
    def destroy(self):
        raise NotImplementedError("Should be overridden by subclasses")



"""
   Copyright 2006-2008 SpringSource (http://springsource.com), All Rights Reserved

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.       
"""
import logging
from springpython.database import DataAccessException
from springpython.security import AuthenticationException
from springpython.security import AuthenticationServiceException
from springpython.security import BadCredentialsException
from springpython.security import DisabledException
from springpython.security import UsernameNotFoundException
from springpython.security.providers import AuthenticationProvider
from springpython.security.providers import UsernamePasswordAuthenticationToken
from springpython.security.providers.encoding import PlaintextPasswordEncoder

class UserCache(object):
    def get_user(self, username):
        raise NotImplementedError()

    def put_user(self, user):
        raise NotImplementedError()

    def remove_user(self, username):
        raise NotImplementedError()
    
class NullUserCache(UserCache):
    def get_user(self, username):
        return None

    def put_user(self, user):
        pass

    def remove_user(self, username):
        pass

class AbstractUserDetailsAuthenticationProvider(AuthenticationProvider):
    def __init__(self):
        super(AbstractUserDetailsAuthenticationProvider, self).__init__()
        self.user_cache = NullUserCache()
        self.hide_user_not_found_exceptions = True
        self.force_principal_as_str = True
        self.logger = logging.getLogger("springpython.security.providers.AbstractUserDetailsAuthenticationProvider")

    def authenticate(self, authentication):
        # Determine username
        username = authentication.username

        cache_was_used = True
        user = self.user_cache.get_user(username)

        if user is None:
            cache_was_used = False

            try:
                user = self.retrieve_user(username, authentication)
            except UsernameNotFoundException, notFound:
                if self.hide_user_not_found_exceptions:
                    raise BadCredentialsException("UsernameNotFound: Bad credentials")
                else:
                    raise notFound

            if user is None:
                raise Exception("retrieve_user returned null - a violation of the interface contract")

        if not user.accountNonLocked:
            raise LockedException("User account is locked")

        if not user.enabled:
            raise DisabledException("User is disabled")

        if not user.accountNonExpired:
            raise AccountExpiredException("User account has expired")
        
        # This check must come here, as we don't want to tell users
        # about account status unless they presented the correct credentials
        try:
            self.additional_auth_checks(user, authentication)
        except AuthenticationException, exception:
            if cache_was_used:
                # There was a problem, so try again after checking we're using latest data (ie not from the cache)
                cache_was_used = False
                user = self.retrieve_user(username, authentication)
                self.additional_auth_checks(user, authentication)
            else:
                raise exception

        if not user.credentialsNonExpired:
            raise CredentialsExpiredException("User credentials have expired")

        if not cache_was_used:
            self.user_cache.put_user(user)

        principal_to_return = user
        
        if self.force_principal_as_str:
            principal_to_return = user.username

        return self.create_success_auth(principal_to_return, authentication, user)

    def additional_auth_checks(self, user_details, authentication):
        raise NotImplementedError()
    
    def retrieve_user(self, username, authentication):
        raise NotImplementedError()
    
    def create_success_auth(self, principal, authentication, user):
        # Ensure we return the original credentials the user supplied,
        # so subsequent attempts are successful even with encoded passwords.
        # Also ensure we return the original getDetails(), so that future
        # authentication events after cache expiry contain the details
        result = UsernamePasswordAuthenticationToken(principal, authentication.getCredentials(), user.authorities)
        #result.details = authentication.details
        return result

class DaoAuthenticationProvider(AbstractUserDetailsAuthenticationProvider):
    def __init__(self, user_details_service = None, password_encoder = PlaintextPasswordEncoder()):
        super(DaoAuthenticationProvider, self).__init__()
        self.password_encoder = password_encoder
        self.salt_source = None
        self.user_details_service = user_details_service
        self.include_details_obj = True
        self.logger = logging.getLogger("springpython.security.providers.DaoAuthenticationProvider")
        
    def retrieve_user(self, username, authentication):
        loaded_user = None

        try:
            loaded_user = self.user_details_service.load_user(username)
        except DataAccessException, repositoryProblem:
            raise AuthenticationServiceException(repositoryProblem)

        if loaded_user is None:
            raise AuthenticationServiceException("UserDetailsService returned null, which is an interface contract violation")

        return loaded_user

    def additional_auth_checks(self, user_details, authentication):
        salt = None

        if self.salt_source is not None:
            salt = self.salt_source.get_salt(user_details)
            
        if not self.password_encoder.isPasswordValid(user_details.password, authentication.getCredentials(), salt):
            raise BadCredentialsException("additional_auth_checks: Bad credentials")

class SaltSource(object):
    """Provides alternative sources of the salt to use for encoding passwords."""
    
    def get_salt(self, user):
        """Returns the salt to use for the indicated user."""
        raise NotImplementedError()
    
class SystemWideSaltSource(SaltSource):
    """
    Uses a static system-wide String as the salt.

    Does not supply a different salt for each User. This means users sharing the same
    password will still have the same digested password. Of benefit is the digested passwords will at least be more protected than if stored without any salt.
    """
    
    def __init__(self, system_wide_salt = ""):
        super(SystemWideSaltSource, self).__init__()
        self.system_wide_salt = system_wide_salt
        
    def get_salt(self, user):
        return self.system_wide_salt
    
class ReflectionSaltSource(SaltSource):
    """
    Obtains a salt from a specified property of the User object.

    This allows you to subclass User and provide an additional bean getter for a salt.
    You should use a synthetic value that does not change, such as a database primary key.
    Do not use username if it is likely to change.
    """
    
    def __init__(self, user_prop_to_use = ""):
        super(ReflectionSaltSource, self).__init__()
        self.user_prop_to_use = user_prop_to_use
        
    def get_salt(self, user):
        try:
            reflectionMethod = getattr(user, self.user_prop_to_use)
            return reflectionMethod()
        except Exception, e:
            raise AuthenticationServiceException(e);





"""
   Copyright 2006-2008 SpringSource (http://springsource.com), All Rights Reserved

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.       
"""
import logging
import threading
from springpython.security.providers import Authentication

# See GlobalSecurityContextHolderStrategy
_globalContext = None

class SecurityContext(object):
    def __init__(self, authentication = Authentication()):
        self.authentication = authentication

    def __str__(self):
        if self.authentication == None:
            return "Authentication is empty"
        else:
            return self.authentication.__str__()

class SecurityContextHolderStrategy(object):
    """Strategy interface to allow defining ways to store security context."""
    def clearContext(self):
        raise NotImplementedError()
    def getContext(self):
        raise NotImplementedError()
    def setContext(self, context):
        raise NotImplementedError()

class ThreadLocalSecurityContextHolderStrategy(SecurityContextHolderStrategy):
    """Strategy to store the security context in a local thread. This allows multi-threaded
    apps to manage multiple contexts at the same time."""
    def __init__(self):
        self.logger = logging.getLogger("springpython.security.context.ThreadLocalSecurityContextHolderStrategy")
        self.contextHolder = threading.local()
        self.logger.debug("Creating a new threadlocal security context.")
        self.clearContext()

    def clearContext(self):
        self.contextHolder.context = None

    def getContext(self):
        if not self.contextHolder.context:
            self.setContext(SecurityContext())
        return self.contextHolder.context     

    def setContext(self, context):
        if not context:
            raise SecurityException("Only non-None security context's are allowed")
        self.contextHolder.context = context    

class GlobalSecurityContextHolderStrategy(SecurityContextHolderStrategy):
    """Store one context in the entire python virtual machine. This typically suits a client-side
    application."""
    def __init__(self):
        self.logger = logging.getLogger("springpython.security.context.GlobalSecurityContextHolderStrategy")
        self.clearContext()

    def clearContext(self):
        global _globalContext
        _globalContext = None

    def getContext(self):
        global _globalContext
        if not _globalContext:
            self.setContext(SecurityContext())
        return _globalContext

    def setContext(self, context):
        global _globalContext
        if not context:
            raise SecurityException("Only non-None security context's are allowed")
        _globalContext = context



"""
   Copyright 2006-2008 SpringSource (http://springsource.com), All Rights Reserved

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""

try:
    import cElementTree as etree
except ImportError:
    try:
        import xml.etree.ElementTree as etree
    except ImportError:
        from elementtree import ElementTree as etree

import re
import types
import inspect
import logging

from _config_base import *
from springpython.context import scope
from springpython.context import ApplicationContextAware
from springpython.factory import PythonObjectFactory
from springpython.factory import ReflectiveObjectFactory
from springpython.container import InvalidObjectScope


xml_mappings = {
    "str":"types.StringType", "unicode":"types.UnicodeType",
    "int":"types.IntType", "long":"types.LongType",
    "float":"types.FloatType", "decimal":"decimal.Decimal",
    "bool":"types.BooleanType", "complex":"types.ComplexType",
}

class PyContainerConfig(Config):
    """
    PyContainerConfig supports the legacy XML dialect (PyContainer) of reading object definitions.
    """

    NS = "{http://www.springframework.org/springpython/schema/pycontainer-components}"

    def __init__(self, config_location):
        if isinstance(config_location, list):
            self.config_location = config_location
        else:
            self.config_location = [config_location]
        self.logger = logging.getLogger("springpython.config.PyContainerConfig")

    def read_object_defs(self):
        self.logger.debug("==============================================================")
        objects = []
        for config in self.config_location:
            self.logger.debug("* Parsing %s" % config)
            components = etree.parse(config).getroot()
            objects.extend([self._convert_component(component) for component in components])
        self.logger.debug("==============================================================")
        return objects


    def _convert_component(self, component):
        "This function generates a object definition, then converts scope and property elements."
        self.logger.debug("component: Processing %s" % component)
        c = ObjectDef(component.get("id"), factory=ReflectiveObjectFactory(component.get("class")))
        if "scope" in component.attrib:
            c.scope = scope.convert(component.get("scope"))
        c.props = [self._convert_prop_def(p) for p in component.findall(self.NS+"property")]
        return c

    def _convert_prop_def(self, p):
        "This function translates object properties into useful dictionaries of information for the container."
        if "local" in p.attrib or p.find(self.NS+"local") is not None:
            if "local" in p.attrib:
                return ReferenceDef(p.get("name"), p.get("local"))
            else:
                return ReferenceDef(p.get("name"), p.find(self.NS+"local"))
        elif "list" in p.attrib or p.find(self.NS+"list") is not None:
            if "list" in p.attrib:
                return ListDef(p.name, [ReferenceDef(p.name + ".list", prop_list.local) for prop_list in p.list])
            else:
                return ListDef(p.name, [ReferenceDef(p.name + ".list", prop_list.local) for prop_list in p.list])
        else:
            self.logger.debug("py: name = %s code = %s" % (p.get("name"), p.text))
            thing = eval(str(p.text).strip())
            self.logger.debug("py: You have parsed %s" % thing)
            return ValueDef(p.get("name"), eval(str(p.text).strip()))

class SpringJavaConfig(Config):
    """
    SpringJavaConfig supports current Spring Java format of XML bean definitions.
    """
    NS = "{http://www.springframework.org/schema/beans}"

    def __init__(self, config_location):
        if isinstance(config_location, list):
            self.config_location = config_location
        else:
            self.config_location = [config_location]
        self.logger = logging.getLogger("springpython.config.SpringJavaConfig")

        # By making this an instance-based property (instead of function local), inner object
        # definitions can add themselves to the list in the midst of parsing an input.
        self.objects = []

    def read_object_defs(self):
        self.logger.debug("==============================================================")
        # Reset, in case the file is re-read
        self.objects = []
        for config in self.config_location:
            self.logger.debug("* Parsing %s" % config)
            beans = etree.parse(config).getroot()
            self.objects.extend([self._convert_bean(bean) for bean in beans])
        self.logger.debug("==============================================================")
        return self.objects

    def _convert_bean(self, bean, prefix=""):
        "This function generates a object definition, then converts scope and property elements."
        if prefix != "":
            if "id" in bean.attrib:
                bean.set("id", prefix + bean.get("id"))
            else:
                bean.set("id", prefix + "<anonymous>")

        c = ObjectDef(bean.get("id"), factory=ReflectiveObjectFactory(bean.get("class")))

        if "scope" in bean.attrib:
            c.scope = scope.convert(bean.get("scope"))
        self.logger.debug("bean: %s" % bean)
        c.pos_constr = [self._convert_prop_def(bean, constr, bean.get("id") + ".constr") for constr in bean.findall(self.NS+"constructor-arg")]
        self.logger.debug("Constructors = %s" % c.pos_constr)
        c.props = [self._convert_prop_def(bean, p, p.get("name")) for p in bean.findall(self.NS+"property")]

        return c

    def _convert_prop_def(self, bean, p, name):
        "This function translates object constructors/properties into useful collections of information for the container."

        if "ref" in p.keys() or p.find(self.NS+"ref") is not None:
            if "ref" in p.keys():
                return ReferenceDef(name, p.get("ref"))
            else:
                return ReferenceDef(name, p.find(self.NS+"ref").get("bean"))
        elif "value" in p.keys() or p.find(self.NS+"value") is not None:
            if "value" in p.keys():
                return ValueDef(name, p.get("value"))
            else:
                return ValueDef(name, p.find(self.NS+"value").text)
        elif p.find(self.NS+"map") is not None:
            dict = {}
            for entry in p.find(self.NS+"map"):
                key = entry.find(self.NS+"key").find(self.NS+"value").text
                if entry.find(self.NS+"value") is not None:
                    dict[key] = str(entry.find(self.NS+"value").text)
                elif entry.find(self.NS+"ref") is not None:
                    dict[key] = ReferenceDef(key, entry.find(self.NS+"ref").get("bean"))
                else:
                    self.logger.debug("Don't know how to handle %s" % entry)
            return DictDef(name, dict)
        elif p.find(self.NS+"props") is not None:
            dict = {}
            for prop in p.find(self.NS+"props"):
                dict[prop.get("key")] = str(prop.text)
            return DictDef(name, dict)
        elif p.find(self.NS+"list") is not None:
            list = []
            for element in p.find(self.NS+"list"):
                if element.tag == self.NS+"value":
                    list.append(element.text)
                elif element.tag == self.NS+"ref":
                    list.append(ReferenceDef(name + ".list", element.get("bean")))
                else:
                    self.logger.debug("Don't know how to handle %s" % element.tag)
            return ListDef(name, list)
        elif p.find(self.NS+"set") is not None:
            s = set()
            for element in p.find(self.NS+"set"):
                if element.tag == self.NS+"value":
                    s.add(element.text)
                elif element.tag == self.NS+"ref":
                    s.add(ReferenceDef(name + ".set", element.get("bean")))
                else:
                    self.logger.debug("Don't know how to handle %s" % element.tag)
            return SetDef(name, s)
        elif p.find(self.NS+"bean"):
            inner_object_def = self._convert_bean(p.find(self.NS+"bean"), prefix=bean.get("id") + "." + name + ".")
            self.objects.append(inner_object_def)
            return InnerObjectDef(name, inner_object_def)

class XMLConfig(Config):
    """
    XMLConfig supports current Spring Python format of XML object definitions.
    """

    NS = "{http://www.springframework.org/springpython/schema/objects}"
    NS_11 = "{http://www.springframework.org/springpython/schema/objects/1.1}"

    def __init__(self, config_location):
        if isinstance(config_location, list):
            self.config_location = config_location
        else:
            self.config_location = [config_location]
        self.logger = logging.getLogger("springpython.config.XMLConfig")

        # By making this an instance-based property (instead of function local), inner object
        # definitions can add themselves to the list in the midst of parsing an input.
        self.objects = []

    def read_object_defs(self):
        self.logger.debug("==============================================================")
        # Reset, in case the file is re-read
        self.objects = []
        for config in self.config_location:
            self.logger.debug("* Parsing %s" % config)

            # A flat list of objects, as found in the XML document.
            objects = etree.parse(config).getroot()

            # We need to handle both 1.0 and 1.1 XSD schemata *and* we may be
            # passed a list of config locations of different XSD versions so we
            # must find out here which one is used in the current config file
            # and pass the correct namespace down to other parts of XMLConfig.
            ns = objects.tag[:objects.tag.find("}") + 1]

            # A dictionary of abstract objects, keyed by their IDs, used in
            # traversing the hierarchies of parents; built upfront here for
            # convenience.
            abstract_objects = {}
            for obj in objects:
                if obj.get("abstract"):
                    abstract_objects[obj.get("id")] = obj

            for obj in objects:
                if obj.get("class") is None and not obj.get("parent"):
                    self._map_custom_class(obj, xml_mappings, ns)

                elif obj.get("parent"):
                    # Children are added to self.objects during the children->abstract parents traversal.
                    pos_constr = self._get_pos_constr(obj, ns)
                    named_constr = self._get_named_constr(obj, ns)
                    props = self._get_props(obj, ns)
                    self._traverse_parents(obj, obj, ns, pos_constr, named_constr, props, abstract_objects)
                    continue

                self.objects.append(self._convert_object(obj, ns=ns))

        self.logger.debug("==============================================================")
        for object in self.objects:
            self.logger.debug("Parsed %s" % object)
        return self.objects

    def _map_custom_class(self, obj, mappings, ns):
        """ Fill in the missing attributes of Python objects and make it look
        to the rest of XMLConfig as if they already were in the XML config file.
        """
        for class_name in mappings:
            tag_no_ns = obj.tag.replace(ns, "")
            if class_name == tag_no_ns:

                obj.set("class", mappings[class_name])
                constructor_arg = etree.Element("%s%s" % (ns, "constructor-arg"))
                value = etree.Element("%s%s" % (ns, "value"))
                value.text = obj.text
                obj.append(constructor_arg)
                constructor_arg.append(value)
                obj.text = ""

                break

        else:
            self.logger.warning("No matching type found for object %s" % obj)

    def _traverse_parents(self, leaf, child, ns, pos_constr,
                            named_constr, props, abstract_objects):

        parent = abstract_objects[child.get("parent")]

        # At this point we only build up the lists of parameters but we don't create
        # the object yet because the current parent object may still have its
        # own parent.

        # Positional constructors

        parent_pos_constrs = self._get_pos_constr(parent, ns)

        # Make sure there are as many child positional parameters as there
        # are in the parent's list.

        len_pos_constr = len(pos_constr)
        len_parent_pos_constrs = len(parent_pos_constrs)

        if len_pos_constr < len_parent_pos_constrs:
            pos_constr.extend([None] * (len_parent_pos_constrs - len_pos_constr))

        for idx, parent_pos_constr in enumerate(parent_pos_constrs):
            if not pos_constr[idx]:
                pos_constr[idx] = parent_pos_constr

        # Named constructors
        child_named_constrs = named_constr
        parent_named_constrs = self._get_named_constr(parent, ns)

        for parent_named_constr in parent_named_constrs:
            if parent_named_constr not in child_named_constrs:
                named_constr[parent_named_constr] = parent_named_constrs[parent_named_constr]

        # Properties
        child_props = [prop.name for prop in props]
        parent_props = self._get_props(parent, ns)

        for parent_prop in parent_props:
            if parent_prop.name not in child_props:
                props.append(parent_prop)

        if parent.get("parent"):
            self._traverse_parents(leaf, parent, ns, pos_constr, named_constr, props, abstract_objects)
        else:
            # Now we know we can create an object out of all the accumulated values.

            # The object's class is its topmost parent's class.
            class_ = parent.get("class")
            id, factory, lazy_init, abstract, parent, scope_ = self._get_basic_object_data(leaf, class_)

            c = self._create_object(id, factory, lazy_init, abstract, parent,
                           scope_, pos_constr, named_constr, props)

            self.objects.append(c)

        return parent

    def _get_pos_constr(self, object, ns):
        """ Returns a list of all positional constructor arguments of an object.
        """
        return [self._convert_prop_def(object, constr, object.get("id") + ".constr", ns) for constr in object.findall(ns+"constructor-arg")
                if not "name" in constr.attrib]

    def _get_named_constr(self, object, ns):
        """ Returns a dictionary of all named constructor arguments of an object.
        """
        return dict([(str(constr.get("name")), self._convert_prop_def(object, constr, object.get("id") + ".constr", ns))
                    for constr in object.findall(ns+"constructor-arg")  if "name" in constr.attrib])

    def _get_props(self, object, ns):
        """ Returns a list of all properties defined by an object.
        """
        return [self._convert_prop_def(object, p, p.get("name"), ns) for p in object.findall(ns+"property")]

    def _create_object(self, id, factory, lazy_init, abstract, parent,
                           scope, pos_constr, named_constr, props):
        """ A helper function which creates an object out of the supplied
        arguments.
        """

        c = ObjectDef(id=id, factory=factory, lazy_init=lazy_init,
            abstract=abstract, parent=parent)

        c.scope = scope
        c.pos_constr = pos_constr
        c.named_constr = named_constr
        c.props = props

        self.logger.debug("object: props = %s" % c.props)
        self.logger.debug("object: There are %s props" % len(c.props))

        return c

    def _get_basic_object_data(self, object, class_):
        """ A convenience method which creates basic object's data so that
        the code is not repeated.
        """

        if "scope" in object.attrib:
            scope_ = scope.convert(object.get("scope"))
        else:
            scope_ = scope.SINGLETON

        return(object.get("id"),  ReflectiveObjectFactory(class_),
            object.get("lazy-init", False), object.get("abstract", False),
               object.get("parent"), scope_)

    def _convert_object(self, object, prefix="", ns=None):
        """ This function collects all parameters required for an object creation
        and then calls a helper function which creates it.
        """
        if prefix != "":
            if "id" in object.attrib:
                object.set("id", prefix + "." + object.get("id"))
            else:
                object.set("id", prefix + ".<anonymous>")

        id, factory, lazy_init, abstract, parent, scope_ = self._get_basic_object_data(object, object.get("class"))

        pos_constr = self._get_pos_constr(object, ns)
        named_constr = self._get_named_constr(object, ns)
        props = self._get_props(object, ns)

        return self._create_object(id, factory, lazy_init, abstract, parent,
            scope_, pos_constr, named_constr, props)

    def _convert_ref(self, ref_node, name):
        if hasattr(ref_node, "attrib"):
            results = ReferenceDef(name, ref_node.get("object"))
            self.logger.debug("ref: Returning %s" % results)
            return results
        else:
            results = ReferenceDef(name, ref_node)
            self.logger.debug("ref: Returning %s" % results)
            return results

    def _convert_value(self, value, id, name, ns):
        if value.text is not None and value.text.strip() != "":
            self.logger.debug("value: Converting a direct value <%s>" % value.text)
            return value.text
        else:
            if value.tag == ns+"value":
                self.logger.debug("value: Converting a value's children %s" % value.getchildren()[0])
                results = self._convert_value(value.getchildren()[0], id, name, ns)
                self.logger.debug("value: results = %s" % str(results))
                return results
            elif value.tag == ns+"tuple":
                self.logger.debug("value: Converting a tuple")
                return self._convert_tuple(value, id, name, ns).value
            elif value.tag == ns+"list":
                self.logger.debug("value: Converting a list")
                return self._convert_list(value, id, name, ns).value
            elif value.tag == ns+"dict":
                self.logger.debug("value: Converting a dict")
                return self._convert_dict(value, id, name, ns).value
            elif value.tag == ns+"set":
                self.logger.debug("value: Converting a set")
                return self._convert_set(value, id, name, ns).value
            elif value.tag == ns+"frozenset":
                self.logger.debug("value: Converting a frozenset")
                return self._convert_frozen_set(value, id, name, ns).value
            else:
                self.logger.debug("value: %s.%s Don't know how to handle %s" % (id, name, value.tag))

    def _convert_dict(self, dict_node, id, name, ns):
        dict = {}
        for entry in dict_node.findall(ns+"entry"):
            self.logger.debug("dict: entry = %s" % entry)
            key = entry.find(ns+"key").find(ns+"value").text
            self.logger.debug("dict: key = %s" % key)
            if entry.find(ns+"value") is not None:
                dict[key] = self._convert_value(entry.find(ns+"value"), id, "%s.dict['%s']" % (name, key), ns)
            elif entry.find(ns+"ref") is not None:
                dict[key] = self._convert_ref(entry.find(ns+"ref"), "%s.dict['%s']" % (name, key))
            elif entry.find(ns+"object") is not None:
                self.logger.debug("dict: Parsing an inner object definition...")
                dict[key] = self._convert_inner_object(entry.find(ns+"object"), id, "%s.dict['%s']" % (name, key), ns)
            else:
                for token in ["dict", "tuple", "set", "frozenset", "list"]:
                    if entry.find(ns+token) is not None:
                        dict[key] = self._convert_value(entry.find(ns+token), id, "%s.dict['%s']" % (name, key), ns)
                        break
                if key not in dict:
                    self.logger.debug("dict: Don't know how to handle %s" % entry.tag)

        self.logger.debug("Dictionary is now %s" % dict)
        return DictDef(name, dict)

    def _convert_props(self, props_node, name, ns):
        dict = {}
        self.logger.debug("props: Looking at %s" % props_node)
        for prop in props_node:
            dict[prop.get("key")] = str(prop.text)
        self.logger.debug("props: Dictionary is now %s" % dict)
        return DictDef(name, dict)

    def _convert_list(self, list_node, id, name, ns):
        list = []
        self.logger.debug("list: Parsing %s" % list_node)
        for element in list_node:
            if element.tag == ns+"value":
                list.append(get_string(element.text))
            elif element.tag == ns+"ref":
                list.append(self._convert_ref(element, "%s.list[%s]" % (name, len(list))))
            elif element.tag == ns+"object":
                self.logger.debug("list: Parsing an inner object definition...")
                list.append(self._convert_inner_object(element, id, "%s.list[%s]" % (name, len(list)), ns))
            elif element.tag in [ns+token for token in ["dict", "tuple", "set", "frozenset", "list"]]:
                self.logger.debug("This list has child elements of type %s." % element.tag)
                list.append(self._convert_value(element, id, "%s.list[%s]" % (name, len(list)), ns))
                self.logger.debug("List is now %s" % list)
            else:
                self.logger.debug("list: Don't know how to handle %s" % element.tag)
        self.logger.debug("List is now %s" % list)
        return ListDef(name, list)

    def _convert_tuple(self, tuple_node, id, name, ns):
        list = []
        self.logger.debug("tuple: Parsing %s" % tuple_node)
        for element in tuple_node:
            self.logger.debug("tuple: Looking at %s" % element)
            if element.tag == ns+"value":
                self.logger.debug("tuple: Appending %s" % element.text)
                list.append(get_string(element.text))
            elif element.tag == ns+"ref":
                list.append(self._convert_ref(element, "%s.tuple(%s}" % (name, len(list))))
            elif element.tag == ns+"object":
                self.logger.debug("tuple: Parsing an inner object definition...")
                list.append(self._convert_inner_object(element, id, "%s.tuple(%s)" % (name, len(list)), ns))
            elif element.tag in [ns+token for token in ["dict", "tuple", "set", "frozenset", "list"]]:
                self.logger.debug("tuple: This tuple has child elements of type %s." % element.tag)
                list.append(self._convert_value(element, id, "%s.tuple(%s)" % (name, len(list)), ns))
                self.logger.debug("tuple: List is now %s" % list)
            else:
                self.logger.debug("tuple: Don't know how to handle %s" % element.tag)
        self.logger.debug("Tuple is now %s" % str(tuple(list)))
        return TupleDef(name, tuple(list))

    def _convert_set(self, set_node, id, name, ns):
        s = set()
        self.logger.debug("set: Parsing %s" % set_node)
        for element in set_node:
            self.logger.debug("Looking at element %s" % element)
            if element.tag == ns+"value":
                s.add(get_string(element.text))
            elif element.tag == ns+"ref":
                s.add(self._convert_ref(element, name + ".set"))
            elif element.tag == ns+"object":
                self.logger.debug("set: Parsing an inner object definition...")
                s.add(self._convert_inner_object(element, id, "%s.set(%s)" % (name, len(s)), ns))
            elif element.tag in [ns+token for token in ["dict", "tuple", "set", "frozenset", "list"]]:
                self.logger.debug("set: This set has child elements of type %s." % element.tag)
                s.add(self._convert_value(element, id, "%s.set(%s)" % (name,len(s)), ns))
            else:
                self.logger.debug("set: Don't know how to handle %s" % element.tag)
        self.logger.debug("Set is now %s" % s)
        return SetDef(name, s)

    def _convert_frozen_set(self, frozen_set_node, id, name, ns):
        item = self._convert_set(frozen_set_node, id, name, ns)
        self.logger.debug("frozenset: Frozen set is now %s" % frozenset(item.value))
        return FrozenSetDef(name, frozenset(item.value))

    def _convert_inner_object(self, object_node, id, name, ns):
        inner_object_def = self._convert_object(object_node, prefix="%s.%s" % (id, name), ns=ns)
        self.logger.debug("innerobj: Innerobject is now %s" % inner_object_def)
        self.objects.append(inner_object_def)
        return InnerObjectDef(name, inner_object_def)

    def _convert_prop_def(self, comp, p, name, ns):
        "This function translates object properties into useful collections of information for the container."
        #self.logger.debug("Is %s.%s a ref? %s" % (comp.get("id"), p.get("name"), p.find(ns+"ref") is not None or "ref" in p.attrib))
        #self.logger.debug("Is %s.%s a value? %s" % (comp.get("id"), p.get("name"), p.find(ns+"value") is not None or "value" in p.attrib))
        #self.logger.debug("Is %s.%s an inner object? %s" % (comp.get("id"), p.get("name"), p.find(ns+"object") is not None or "object" in p.attrib))
        #self.logger.debug("Is %s.%s a dict? %s" % (comp.get("id"), p.get("name"), p.find(ns+"dict") is not None or "dict" in p.attrib))
        #self.logger.debug("Is %s.%s a list? %s" % (comp.get("id"), p.get("name"), p.find(ns+"list") is not None or "list" in p.attrib))
        #self.logger.debug("Is %s.%s a tuple? %s" % (comp.get("id"), p.get("name"), p.find(ns+"tuple") is not None or "tuple" in p.attrib))
        #self.logger.debug("Is %s.%s a set? %s" % (comp.get("id"), p.get("name"), p.find(ns+"set") is not None or "set" in p.attrib))
        #self.logger.debug("Is %s.%s a frozenset? %s" % (comp.get("id"), p.get("name"), p.find(ns+"frozenset") is not None or "frozenset" in p.attrib))
        #self.logger.debug("")
        if "ref" in p.attrib or p.find(ns+"ref") is not None:
            if "ref" in p.attrib:
                return self._convert_ref(p.get("ref"), name)
            else:
                return self._convert_ref(p.find(ns+"ref"), name)
        elif "value" in p.attrib or p.find(ns+"value") is not None:
            if "value" in p.attrib:
                return ValueDef(name, get_string(p.get("value")))
            else:
                return ValueDef(name, get_string(p.find(ns+"value").text))
        elif "dict" in p.attrib or p.find(ns+"dict") is not None:
            if "dict" in p.attrib:
                return self._convert_dict(p.get("dict"), comp.get("id"), name, ns)
            else:
                return self._convert_dict(p.find(ns+"dict"), comp.get("id"), name, ns)
        elif "props" in p.attrib or p.find(ns+"props") is not None:
            if "props" in p.attrib:
                return self._convert_props(p.get("props"), name, ns)
            else:
                return self._convert_props(p.find(ns+"props"), name, ns)
        elif "list" in p.attrib or p.find(ns+"list") is not None:
            if "list" in p.attrib:
                return self._convert_list(p.get("list"), comp.get("id"), name, ns)
            else:
                return self._convert_list(p.find(ns+"list"), comp.get("id"), name, ns)
        elif "tuple" in p.attrib or p.find(ns+"tuple") is not None:
            if "tuple" in p.attrib:
                return self._convert_tuple(p.get("tuple"), comp.get("id"), name, ns)
            else:
                return self._convert_tuple(p.find(ns+"tuple"), comp.get("id"), name, ns)
        elif "set" in p.attrib or p.find(ns+"set") is not None:
            if "set" in p.attrib:
                return self._convert_set(p.get("set"), comp.get("id"), name, ns)
            else:
                return self._convert_set(p.find(ns+"set"), comp.get("id"), name, ns)
        elif "frozenset" in p.attrib or p.find(ns+"frozenset") is not None:
            if "frozenset" in p.attrib:
                return self._convert_frozen_set(p.get("frozenset"), comp.get("id"), name, ns)
            else:
                return self._convert_frozen_set(p.find(ns+"frozenset"), comp.get("id"), name, ns)
        elif "object" in p.attrib or p.find(ns+"object") is not None:
            if "object" in p.attrib:
                return self._convert_inner_object(p.get("object"), comp.get("id"), name, ns)
            else:
                return self._convert_inner_object(p.find(ns+"object"), comp.get("id"), name, ns)




"""
   Copyright 2006-2008 SpringSource (http://springsource.com), All Rights Reserved

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.       
"""
import logging

class PasswordEncoder(object):
    """Interface for performing authentication operations on a password."""
    
    def encodePassword(self, rawPass, salt):
        """Encodes the specified raw password with an implementation specific algorithm."""
        raise NotImplementedError()
    
    def isPasswordValid(self, encPass, rawPass, salt):
        """Validates a specified "raw" password against an encoded password."""
        raise NotImplementedError()
    
class BasePasswordEncoder(PasswordEncoder):
    """Convenience base for all password encoders."""
    
    def __init__(self):
        super(BasePasswordEncoder, self).__init__()
        self.ignorePasswordCase = False
        self.logger = logging.getLogger("springpython.security.providers.encoding.BasePasswordEncoder")

    def mergePasswordAndSalt(self, password, salt, strict):
        """
        Used by subclasses to generate a merged password and salt String.
        The generated password will be in the form of 'password{salt}'.
        A None can be passed to either parameter, and will be handled correctly. If the salt is None or empty,
        the resulting generated password will simply be the passed password. The __str__ method of the salt will be used to represent the salt.
        """
        if password is None:
            password = ""

        if strict and salt is not None:
            if "{" in str(salt) or "}" in str(salt):
                raise ValueError("Cannot use { or } in salt.__str__")

        if salt is None or salt == "":
            return password
        else:
            return password + "{" + str(salt) + "}"


class PlaintextPasswordEncoder(BasePasswordEncoder):
    """
    Plaintext implementation of PasswordEncoder.

    As callers may wish to extract the password and salts separately from the encoded password,
    the salt must not contain reserved characters (specifically '{' and '}').
    """

    def __init__(self):
        super(PlaintextPasswordEncoder, self).__init__()
        self.logger = logging.getLogger("springpython.security.providers.encoding.PlaintextPasswordEncoder")
        
    def encodePassword(self, rawPass, salt):
        """Encodes the specified raw password with an implementation specific algorithm."""
        return self.mergePasswordAndSalt(rawPass, salt, True)

    def isPasswordValid(self, encPass, rawPass, salt):
        """Validates a specified "raw" password against an encoded password."""
        pass1 = encPass + ""

        # Strict delimiters is false because pass2 never persisted anywhere
        # and we want to avoid unnecessary exceptions as a result (the
        # authentication will fail as the encodePassword never allows them)
        pass2 = self.mergePasswordAndSalt(rawPass, salt, False)
        
        if not self.ignorePasswordCase:
            return pass1 == pass2
        else:
            return pass1.upper() == pass2.upper()

class AbstractOneWayPasswordEncoder(BasePasswordEncoder):
    """
    This is an abstract one-way hashing encoder. It is abstract because the
    subclasses have to plugin their strategy.
    """
    def __init__(self):
        super(AbstractOneWayPasswordEncoder, self).__init__()
        self.onewayHasher = None
        self.logger = logging.getLogger("springpython.security.providers.encoding.AbstractOneWayPasswordEncoder")
        
    def encodePassword(self, rawPass, salt):
        """Encodes the specified raw password with an implementation specific algorithm."""
        hasher = self.onewayHashStrategy()
        if not self.ignorePasswordCase:
            hasher.update(self.mergePasswordAndSalt(rawPass, salt, False))
        else:
            hasher.update(self.mergePasswordAndSalt(rawPass.lower(), salt, False))
        return hasher.hexdigest()

    def isPasswordValid(self, encPass, rawPass, salt):
        """Validates a specified "raw" password against an encoded password."""
        pass1 = encPass + ""

        # Strict delimiters is false because pass2 never persisted anywhere
        # and we want to avoid unnecessary exceptions as a result (the
        # authentication will fail as the encodePassword never allows them)
        pass2 = self.mergePasswordAndSalt(rawPass, salt, False)

        hasher = self.onewayHashStrategy()
        if self.ignorePasswordCase:
            hasher.update(pass2.lower())
        else:
            hasher.update(pass2)
        pass2 = hasher.hexdigest()
        
        if not self.ignorePasswordCase:
            return pass1 == hasher.hexdigest()
        else:
            return pass1.lower() == hasher.hexdigest()

class Md5PasswordEncoder(AbstractOneWayPasswordEncoder):
    """
    MD5 implementation of PasswordEncoder.
    
    If a None password is presented, it will be treated as an empty String ("") password.
    
    As MD5 is a one-way hash, the salt can contain any characters.
    """

    def __init__(self):
        super(Md5PasswordEncoder, self).__init__()
        try:
            import hashlib
            self.onewayHashStrategy = hashlib.md5
        except ImportError:
            import md5
            self.onewayHashStrategy = md5.new
        self.logger = logging.getLogger("springpython.security.providers.encoding.Md5PasswordEncoder")
        
class ShaPasswordEncoder(AbstractOneWayPasswordEncoder):
    """
    SHA implementation of PasswordEncoder.
    
    If a None password is presented, it will be treated as an empty String ("") password.
    
    As SHA is a one-way hash, the salt can contain any characters.
    """

    def __init__(self):
        super(ShaPasswordEncoder, self).__init__()
        try:
            import hashlib
            self.onewayHashStrategy = hashlib.sha1
        except ImportError:
            import sha
            self.onewayHashStrategy = sha.new
        self.logger = logging.getLogger("springpython.security.providers.encoding.ShaPasswordEncoder")


class LdapShaPasswordEncoder(PasswordEncoder):
    def __init__(self):
        super(PasswordEncoder, self).__init__()
        self.sha_encoder = ShaPasswordEncoder()
        self.logger = logging.getLogger("springpython.security.providers.encoding.LdapShaPasswordEncoder")

    def encodePassword(self, rawPass, salt):
        """Encodes the specified raw password with an implementation specific algorithm."""
        import base64
        hasher = self.sha_encoder.onewayHashStrategy()
        hasher.update(rawPass)
        return "{SHA}" + base64.b64encode(hasher.digest())
    
    def isPasswordValid(self, encPass, rawPass, salt):
        """Validates a raw password against an encrypted one. It checks the prefix, to tell if its encrypted
           or stored in the clear."""

        if encPass.startswith("{SHA}"):
            return encPass == self.encodePassword(rawPass, salt)
        else:
            return encPass == rawPass
                




#
# A Hessian client interface for Python.  The date and long types require
# Python 2.2 or later.
#
# The Hessian proxy is used as follows:
#
# proxy = Hessian("http://hessian.caucho.com/test/basic")
#
# print proxy.hello()
#
# --------------------------------------------------------------------
#
# The Apache Software License, Version 1.1
#
# Copyright (c) 2001-2002 Caucho Technology, Inc.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. The end-user documentation included with the redistribution, if
#    any, must include the following acknowlegement:
#       "This product includes software developed by the
#        Caucho Technology (http://www.caucho.com/)."
#    Alternately, this acknowlegement may appear in the software itself,
#    if and wherever such third-party acknowlegements normally appear.
#
# 4. The names "Hessian", "Resin", and "Caucho" must not be used to
#    endorse or promote products derived from this software without prior
#    written permission. For written permission, please contact
#    info@caucho.com.
#
# 5. Products derived from this software may not be called "Resin"
#    nor may "Resin" appear in their names without prior written
#    permission of Caucho Technology.
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED.  IN NO EVENT SHALL CAUCHO TECHNOLOGY OR ITS CONTRIBUTORS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
# OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# --------------------------------------------------------------------
#
# Credits: hessianlib.py was inspired and partially based on
# xmlrpclib.py created by Fredrik Lundh at www.pythonware.org
#
import string, time
import urllib
from types import *
from struct import unpack
from struct import pack

__version__ = "0.1"


# --------------------------------------------------------------------
# Exceptions

class Error:
    # base class for client errors
    pass

class ProtocolError(Error):
    # Represents an HTTP protocol error
    def __init__(self, url, code, message, headers):
	self.url = url
	self.code = code
	self.message = message
	self.headers = headers

    def __repr__(self):
	return (
	    "<ProtocolError for %s: %s %s>" %
	    (self.url, self.code, self.message)
	    )

class Fault(Error):
    # Represents a fault from Hessian
    def __init__(self, code, message, **detail):
	self.code = code
	self.message = message

    def __repr__(self):
	return "<HessianFault %s: %s>" % (self.code, self.message)

# --------------------------------------------------------------------
# Wrappers for Hessian data types non-standard in Python
#

#
# Boolean -- use the True or False constants
#
class Boolean:
    def __init__(self, value = 0):
	self.value = (value != 0)

    def _hessian_write(self, out):
	if self.value:
	    out.write('T')
	else:
	    out.write('F')

    def __repr__(self):
	if self.value:
	    return "<True at %x>" % id(self)
	else:
	    return "<False at %x>" % id(self)

    def __int__(self):
	return self.value

    def __nonzero__(self):
	return self.value

True, False = Boolean(1), Boolean(0)

#
# Date - wraps a time value in seconds
#
class Date:
    def __init__(self, value = 0):
	self.value = value

    def __repr__(self):
	return ("<Date %s at %x>" %
                (time.asctime(time.localtime(self.value)), id(self)))

    def _hessian_write(self, out):
	out.write("d")
	out.write(pack(">q", self.value * 1000.0))
#
# Binary - binary data
#

class Binary:
    def __init__(self, data=None):
	self.data = data

    def _hessian_write(self, out):
	out.write('B')
	out.write(pack('>H', len(self.data)))
	out.write(self.data)

# --------------------------------------------------------------------
# Marshalling and unmarshalling code

#
# HessianWriter - writes Hessian data from Python objects
#
class HessianWriter:
    dispatch = {}

    def write_call(self, method, params):
	self.refs = {}
	self.ref = 0
	self.__out = []
	self.write = write = self.__out.append

        write("c\x01\x00m");
        write(pack(">H", len(method)));
        write(method);
	for v in params:
	    self.write_object(v)
        write("z");
	result = string.join(self.__out, "")
	del self.__out, self.write, self.refs
	return result

    def write_object(self, value):
	try:
	    f = self.dispatch[type(value)]
	except KeyError:
	    raise TypeError, "cannot write %s objects" % type(value)
	else:
	    f(self, value)

    def write_int(self, value):
	self.write('I')
	self.write(pack(">l", value))
    dispatch[IntType] = write_int

    def write_long(self, value):
	self.write('L')
	self.write(pack(">q", value))
    dispatch[LongType] = write_long

    def write_double(self, value):
	self.write('D')
	self.write(pack(">d", value))
    dispatch[FloatType] = write_double

    def write_string(self, value):
	self.write('S')
	self.write(pack('>H', len(value)))
	self.write(value)
    dispatch[StringType] = write_string

    def write_reference(self, value):
        # check for and write circular references
        # returns 1 if the object should be written, i.e. not a reference
	i = id(value)
	if self.refs.has_key(i):
	    self.write('R')
	    self.write(pack(">L", self.refs[i]))
	    return 0
	else:
	    self.refs[i] = self.ref
	    self.ref = self.ref + 1
	    return 1

    def write_list(self, value):
	if self.write_reference(value):
	    self.write("Vt\x00\x00I");
	    self.write(pack('>l', len(value)))
	    for v in value:
	        self.__write(v)
	    self.write('z')
    dispatch[TupleType] = write_list
    dispatch[ListType] = write_list

    def write_map(self, value):
	if self.write_reference(value):
	    self.write("Mt\x00\x00")
	    for k, v in value.items():
	        self.__write(k)
	        self.__write(v)
	    self.write("z")
    dispatch[DictType] = write_map

    def write_instance(self, value):
	# check for special wrappers
	if hasattr(value, "_hessian_write"):
	    value._hessian_write(self)
	else:
	    fields = value.__dict__
	    if self.write_reference(fields):
	        self.write("Mt\x00\x00")
	        for k, v in fields.items():
	            self.__write(k)
	            self.__write(v)
	        self.write("z")
    dispatch[InstanceType] = write_instance

#
# Parses the results from the server
#
class HessianParser:
    def __init__(self, f):
	self._f = f
        self._peek = -1
	# self.read = f.read
	self._refs = []

    def read(self, len):
	if self._peek >= 0:
	  value = self._peek
	  self._peek = -1
	  return value
	else:
	  return self._f.read(len)

    def parse_reply(self):
        # parse header 'c' x01 x00 'v' ... 'z'
	read = self.read
	if read(1) != 'r':
	    self.error()
	major = read(1)
	minor = read(1)

        value = self.parse_object()

	if read(1) == 'z':
	    return value
	self.error() # actually a fault

    def parse_object(self):
	# parse an arbitrary object based on the type in the data
	return self.parse_object_code(self.read(1))

    def parse_object_code(self, code):
	# parse an object when the code is known
	read = self.read

	if code == 'N':
	    return None

	elif code == 'T':
	    return True

	elif code == 'F':
	    return False

	elif code == 'I':
	    return unpack('>l', read(4))[0]

	elif code == 'L':
	    return unpack('>q', read(8))[0]

	elif code == 'D':
	    return unpack('>d', read(8))[0]

	elif code == 'd':
	    ms = unpack('>q', read(8))[0]

	    return Date(int(ms / 1000.0))

	elif code == 'S' or code == 'X':
	    return self.parse_string()

	elif code == 'B':
	    return Binary(self.parse_string())

	elif code == 'V':
	    self.parse_type() # skip type
	    self.parse_length()           # skip length
	    list = []
	    self._refs.append(list)
	    ch = read(1)
	    while ch != 'z':
		list.append(self.parse_object_code(ch))
		ch = read(1)
	    return list

	elif code == 'M':
	    self.parse_type() # skip type
	    map = {}
	    self._refs.append(map)
	    ch = read(1)
	    while ch != 'z':
		key = self.parse_object_code(ch)
		value = self.parse_object()
		map[key] = value
		ch = read(1)
	    return map

	elif code == 'R':
	    return self._refs[unpack('>l', read(4))[0]]

	elif code == 'r':
	    self.parse_type()       # skip type
	    url = self.parse_type() # reads the url
	    return Hessian(url)

	else:
	    raise "UnknownObjectCode %d" % code

    def parse_string(self):
	f = self._f
	len = unpack('>H', f.read(2))[0]
	return f.read(len)

    def parse_type(self):
	f = self._f
	code = self.read(1)
	if code != 't':
	  self._peek = code
	  return ""
	len = unpack('>H', f.read(2))[0]
	return f.read(len)

    def parse_length(self):
	f = self._f
	code = self.read(1);
	if code != 'l':
	  self._peek = code
	  return -1;
	len = unpack('>l', f.read(4))
	return len

    def error(self):
	raise "FOO"

#
# Encapsulates the method to be called
#
class _Method:
    def __init__(self, invoker, method):
	self._invoker = invoker
	self._method = method

    def __call__(self, *args):
	return self._invoker(self._method, args)

# --------------------------------------------------------------------
# Hessian is the main class.  A Hessian proxy is created with the URL
# and then called just as for a local method
#
# proxy = Hessian("http://www.caucho.com/hessian/test/basic")
# print proxy.hello()
#
class Hessian:
    """Represents a remote object reachable by Hessian"""

    def __init__(self, url):
	# Creates a Hessian proxy object

	self._url = url

	# get the uri
	type, uri = urllib.splittype(url)
	if type != "http":
	    raise IOError, "unsupported Hessian protocol"

	self._host, self._uri = urllib.splithost(uri)

    def __invoke(self, method, params):
	# call a method on the remote server

	request = HessianWriter().write_call(method, params)

	import httplib

	h = httplib.HTTP(self._host)
	h.putrequest("POST", self._uri)

	# required by HTTP/1.1
	h.putheader("Host", self._host)

	h.putheader("User-Agent", "hessianlib.py/%s" % __version__)
	h.putheader("Content-Length", str(len(request)))

	h.endheaders()

	h.send(request)

	errcode, errmsg, headers = h.getreply()

	if errcode != 200:
	    raise ProtocolError(self._url, errcode, errmsg, headers)

	return self.parse_response(h.getfile())

    def parse_response(self, f):
	# read response from input file, and parse it

	parser = HessianParser(f)
	value = parser.parse_reply()
	f.close()

	return value

    def _hessian_write(self, out):
	# marshals the proxy itself
	out.write("rt\x00\x00S")
	out.write(pack(">H", len(self._url)))
	out.write(self._url)

    def __repr__(self):
	return "<HessianProxy %s>" % self._url

    __str__ = __repr__

    def __getattr__(self, name):
	# encapsulate the method call
	return _Method(self.__invoke, name)

#
# Testing code.
#
if __name__ == "__main__":

    proxy = Hessian("http://hessian.caucho.com/test/test")

    try:
	print proxy.hello()
    except Error, v:
	print "ERROR", v



"""
   Copyright 2006-2008 SpringSource (http://springsource.com), All Rights Reserved

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.       
"""
import logging
import types
from springpython.database import ArgumentMustBeNamed
from springpython.database import DataAccessException
from springpython.database import IncorrectResultSizeDataAccessException
from springpython.database import InvalidArgumentType
from springpython.database import factory

class DaoSupport(object):
    """
    Any class that extends this one will be provided with a DatabaseTemplate class
    to help carry out database operations. It requires that a connection object be
    provided during instantion.
    """
    def __init__(self, connection_factory = None):
        self.database_template = DatabaseTemplate()
        self.connection_factory = connection_factory
        
    def __setattr__(self, name, value):
        """When the connection factory is set, pass it on through to the database template."""
        self.__dict__[name] = value
        if name == "connection_factory" and value:
            self.__dict__["database_template"].connection_factory = value

class DatabaseTemplate(object):
    """
    This class is meant to mimic the Spring framework's JdbcTemplate class.
    Since Python doesn't use JDBC, the name is generalized to "Database"
    """
    def __init__(self, connection_factory = None):
        self.connection_factory = connection_factory
        self.logger = logging.getLogger("springpython.database.core.DatabaseTemplate")

    def __del__(self):
        "When this template goes out of scope, need to close the connection it formed."
        if self.connection_factory is not None: self.connection_factory.close()
            
    def _execute(self, sql_statement, args = None):
        """Issue a single SQL execute, typically a DDL statement."""
        sql_statement = self.connection_factory.convert_sql_binding(sql_statement)

        cursor = self.connection_factory.getConnection().cursor()
        error = None
        rows_affected = 0
        try:
            try:
                if args:
                    cursor.execute(sql_statement, args)
                    rows_affected = cursor.rowcount
                    lastrowid = cursor.lastrowid
                else:
                    cursor.execute(sql_statement)
                    rows_affected = cursor.rowcount
                    lastrowid = cursor.lastrowid
            except Exception, e:
                self.logger.debug("execute.execute: Trapped %s while trying to execute '%s'" % (e, sql_statement))
                error = e
        finally:
            try:
                cursor.close()
            except Exception, e:
                self.logger.debug("execute.close: Trapped %s, and throwing away." % e)
            
        if error:
            raise DataAccessException(error)
        
        return {"rows_affected":rows_affected, "lastrowid":lastrowid}

    def execute(self, sql_statement, args = None):
        """Execute a single SQL statement, and return the number of rows affected."""
        return self._execute(sql_statement, args)["rows_affected"]

    def insert_and_return_id(self, sql_statement, args = None):
        """Execute a single INSERT statement, and return the PK of the new row."""
        return self._execute(sql_statement, args)["lastrowid"]
    
    def query(self, sql_query, args = None, rowhandler = None):
        """Execute a query given static SQL, reading the ResultSet on a per-row basis with a RowMapper.
        If args is provided, bind the arguments (to avoid SQL injection attacks)."""

        # This is the case where only two, non-named arguments were provided, the sql_query and one other.
        # If the second argument was 'args', it is invalid since 'rowhandler' is required.
        # It is was 'rowhandler', it shifted into 'args' position, and requires naming.
        if args and not rowhandler:
            raise ArgumentMustBeNamed(arg_name="rowhandler")

        results, metadata = self.__query_for_list(sql_query, args)
        return [rowhandler.map_row(row, metadata) for row in results]

    def query_for_list(self, sql_query, args = None):
        results, metadata = self.__query_for_list(sql_query, args)
        return results
    
    def __query_for_list(self, sql_query, args = None):
        """Execute a query for a result list, given static SQL. If args is provided, bind the arguments 
        (to avoid SQL injection attacks)."""

        if args and type(args) not in self.connection_factory.acceptable_types:
            raise InvalidArgumentType(type(args), self.connection_factory.acceptable_types)

        sql_query = self.connection_factory.convert_sql_binding(sql_query)
        
        cursor = self.connection_factory.getConnection().cursor()
        error = None
        results = None
        metadata = None
        try:
            try:
                if args:
                    cursor.execute(sql_query, args)
                else:
                    cursor.execute(sql_query)
                results = cursor.fetchall()
                metadata = [{"name":row[0], "type_code":row[1], "display_size":row[2], "internal_size":row[3], "precision":row[4], "scale":row[5], "null_ok":row[6]} for row in cursor.description]
            except Exception, e:
                self.logger.debug("query_for_list.execute: Trapped %s while trying to execute '%s'" % (e, sql_query))
                error = e
        finally:
            try:
                cursor.close()
            except Exception, e:
                self.logger.debug("query_for_list.close: Trapped %s, and throwing away." % e)

        if error:
            self.logger.debug("query_for_list: I thought about kicking this up the chain => %s" % error)

        # Convert multi-item tuple into list
        return [result for result in results or []], metadata

    def query_for_int(self, sql_query, args = None):
        """Execute a query that results in an int value, given static SQL. If args is provided, bind the arguments 
        (to avoid SQL injection attacks)."""
        return self.query_for_object(sql_query, args, types.IntType)
    
    def query_for_long(self, sql_query, args = None):
        """Execute a query that results in an int value, given static SQL. If args is provided, bind the arguments 
        (to avoid SQL injection attacks)."""
        return self.query_for_object(sql_query, args, types.LongType)
    
    def query_for_object(self, sql_query, args = None, required_type = None):
        """Execute a query that results in an int value, given static SQL. If args is provided, bind the arguments 
        (to avoid SQL injection attacks)."""

        # This is the case where only two, non-named arguments were provided, the sql_query and one other.
        # If the second argument was 'args', it is invalid since 'required_type' is required.
        # It is was 'required_type', it shifted into 'args' position, and requires naming.
        if args and not required_type:
            raise ArgumentMustBeNamed(arg_name="required_type")

        results = self.query_for_list(sql_query, args)
        
        if len(results) != 1:
            raise IncorrectResultSizeDataAccessException("Instead of getting one row, this query returned %s" % len(results))
        
        if len(results[0]) != 1:
            raise IncorrectResultSizeDataAccessException("Instead of getting one column, this query returned %s" % len(results[0]))

        equivalentTypes = [
                           [types.UnicodeType, types.StringType]
                           ]
        if type(results[0][0]) != required_type:
            foundEquivType = False
            for equivType in equivalentTypes:
                if type(results[0][0]) in equivType and required_type in equivType:
                    foundEquivType = True
                    break
            if not foundEquivType:
                raise DataAccessException("Expected %s, but instead got %s"% (required_type, type(results[0][0])))

        return results[0][0]

    def update(self, sql_statement, args = None):
        """Issue a single SQL update.  If args is provided, bind the arguments 
        (to avoid SQL injection attacks)."""
        return self.execute(sql_statement, args)
    
    
class RowMapper(object):
    """
    This is an interface to handle one row of data.
    """
    def map_row(self, row, metadata=None):
        raise NotImplementedError()

class DictionaryRowMapper(RowMapper):
    """
    This row mapper converts the tuple into a dictionary using the column names as the keys.
    """
    def map_row(self, row, metadata=None):
        if metadata is not None:
            obj = {}
            for i, column in enumerate(metadata):
                obj[column["name"]] = row[i]
            return obj
        else:
            raise DataAccessException("metadata is None, unable to convert result set into a dictionary")

class SimpleRowMapper(RowMapper):
    """
    This row mapper uses convention over configuration to create and populate attributes
    of an object.
    """
    def __init__(self, clazz):
        self.clazz = clazz

    def map_row(self, row, metadata=None):
        if metadata is not None:
            obj = self.clazz()
            for i, column in enumerate(metadata):
                setattr(obj, column["name"], row[i])
            return obj
        else:
            raise DataAccessException("metadata is None, unable to map result set into %s instance" % self.clazz)




"""
   Copyright 2006-2008 SpringSource (http://springsource.com), All Rights Reserved

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
import re
import types
import inspect
import logging

from springpython.context import scope
from decorator import decorator, partial
from springpython.context import ApplicationContextAware
from springpython.factory import PythonObjectFactory
from springpython.factory import ReflectiveObjectFactory
from springpython.container import InvalidObjectScope

def get_string(value):
    """This function is used to parse text that could either be ASCII or unicode."""
    try:
        return str(value)
    except UnicodeEncodeError:
        return unicode(value)
        
class ObjectDef(object):
    """
    ObjectDef is a format-neutral way of storing object definition information. It includes
    a handle for the actual ObjectFactory that should be used to utilize this information when
    creating an instance of a object.
    """
    def __init__(self, id, props=None, factory=None, scope=scope.SINGLETON,
                 lazy_init=False, abstract=False, parent=None):
        super(ObjectDef, self).__init__()
        self.id = id
        self.factory = factory
        if props is None:
            self.props = []
        else:
            self.props = props
        self.scope = scope
        self.lazy_init = lazy_init
        self.abstract = abstract
        self.parent = parent
        self.pos_constr = []
        self.named_constr = {}

    def __str__(self):
        return "id=%s props=%s scope=%s factory=%s" % (self.id, self.props, self.scope, self.factory)

class ReferenceDef(object):
    """
    This class represents a definition that is referencing another object.
    """
    def __init__(self, name, ref):
        self.name = name
        self.ref = ref

    def prefetch(self, container):
        self.get_value(container)

    def get_value(self, container):
        return container.get_object(self.ref)

    def set_value(self, obj, container):
        setattr(obj, self.name, container.objects[self.ref])

    def __str__(self):
        return "name=%s ref=%s" % (self.name, self.ref)

class InnerObjectDef(object):
    """
    This class represents an inner object. It is optional whether or not the object
    has its own name.
    """
    def __init__(self, name, inner_comp):
        self.name = name
        self.inner_comp = inner_comp

    def prefetch(self, container):
        self.get_value(container)

    def get_value(self, container):
        return container.get_object(self.inner_comp.id)

    def set_value(self, obj, container):
        setattr(obj, self.name, self.get_value(container))

    def __str__(self):
        return "name=%s inner_comp=%s" % (self.name, self.inner_comp)

class ValueDef(object):
    """
    This class represents a property that holds a value. The value can be simple value, or
    it can be a complex container which internally holds references, inner objects, or
    any other type.
    """
    def __init__(self, name, value):
        self.name = name
        if value == "True":
            self.value = True
        elif value == "False":
            self.value= False
        else:
            self.value = value
        self.logger = logging.getLogger("springpython.config.ValueDef")

    def scan_value(self, container, value):
        if hasattr(value, "get_value"):
            return value.get_value(container)
        elif isinstance(value, tuple):
            new_list = [self.scan_value(container, item) for item in value]
            results = tuple(new_list)
            return results
        elif isinstance(value, list):
            new_list = [self.scan_value(container, item) for item in value]
            return new_list
        elif isinstance(value, set):
            results = set([self.scan_value(container, item) for item in value])
            return results
        elif isinstance(value, frozenset):
            results = frozenset([self.scan_value(container, item) for item in value])
            return results
        else:
            if value == "True":
                return True
            elif value == "False":
                return False
            else:
                return value

    def get_value(self, container):
        val = self._replace_refs_with_actuals(self.value, container)
        if val is None:
            return self.value
        else:
            return val

    def set_value(self, obj, container):
        setattr(obj, self.name, self.value)
        val = self._replace_refs_with_actuals(obj, container)

    def _replace_refs_with_actuals(self, obj, container):
        """Normal values do nothing for this step. However, sub-classes are defined for
        the various containers, like lists, set, dictionaries, etc., to handle iterating
        through and pre-fetching items."""
        pass

    def __str__(self):
        return "name=%s value=%s" % (self.name, self.value)

class DictDef(ValueDef):
    """Handles behavior for a dictionary-based value."""
    def __init__(self, name, value):
        super(DictDef, self).__init__(name, value)

    def _replace_refs_with_actuals(self, obj, container):
        for key in self.value.keys():
            if hasattr(self.value[key], "ref"):
                self.value[key] = container.get_object(self.value[key].ref)
            else:
                self.value[key] = self.scan_value(container, self.value[key])

class ListDef(ValueDef):
    """Handles behavior for a list-based value."""
    def __init__(self, name, value):
        super(ListDef, self).__init__(name, value)
        self.logger = logging.getLogger("springpython.config.ListDef")

    def _replace_refs_with_actuals(self, obj, container):
        for i in range(0, len(self.value)):
            self.logger.debug("Checking out %s, wondering if I need to do any replacement..." % get_string(self.value[i]))
            if hasattr(self.value[i], "ref"):
                self.value[i] = container.get_object(self.value[i].ref)
            else:
                self.value[i] = self.scan_value(container, self.value[i])

class TupleDef(ValueDef):
    """Handles behavior for a tuple-based value."""

    def __init__(self, name, value):
        super(TupleDef, self).__init__(name, value)

    def _replace_refs_with_actuals(self, obj, container):
        new_value = list(self.value)
        for i in range(0, len(new_value)):
            if hasattr(new_value[i], "ref"):
                new_value[i] = container.get_object(new_value[i].ref)
            else:
                new_value[i] = self.scan_value(container, new_value[i])
        try:
            setattr(obj, self.name, tuple(new_value))
        except AttributeError:
            pass
        return tuple(new_value)

class SetDef(ValueDef):
    """Handles behavior for a set-based value."""
    def __init__(self, name, value):
        super(SetDef, self).__init__(name, value)
        self.logger = logging.getLogger("springpython.config.SetDef")

    def _replace_refs_with_actuals(self, obj, container):
        self.logger.debug("Replacing refs with actuals...")
        self.logger.debug("set before changes = %s" % self.value)
        new_set = set()
        for item in self.value:
            if hasattr(item, "ref"):
                self.logger.debug("Item !!!%s!!! is a ref, trying to replace with actual object !!!%s!!!" % (item, item.ref))
                #self.value.remove(item)
                #self.value.add(container.get_object(item.ref))
                newly_fetched_value = container.get_object(item.ref)
                new_set.add(newly_fetched_value)
                self.logger.debug("Item !!!%s!!! was removed, and newly fetched value !!!%s!!! was added." % (item, newly_fetched_value))
                #new_set.add(container.get_object(item.ref))
            else:
                self.logger.debug("Item !!!%s!!! is NOT a ref, trying to replace with scanned value" % get_string(item))
                #self.value.remove(item)
                #self.value.add(self.scan_value(container, item))
                newly_scanned_value = self.scan_value(container, item)
                new_set.add(newly_scanned_value)
                self.logger.debug("Item !!!%s!!! was removed, and newly scanned value !!!%s!!! was added." % (item, newly_scanned_value))
                #new_set.add(self.scan_value(container, item))
        #self.value = new_set
        self.logger.debug("set after changes = %s" % new_set)
        #return self.value
        try:
            setattr(obj, self.name, new_set)
        except AttributeError:
            pass
        return new_set

class FrozenSetDef(ValueDef):
    """Handles behavior for a frozen-set-based value."""
    def __init__(self, name, value):
        super(FrozenSetDef, self).__init__(name, value)
        self.logger = logging.getLogger("springpython.config.FrozenSetDef")

    def _replace_refs_with_actuals(self, obj, container):
        self.logger.debug("Replacing refs with actuals...")
        self.logger.debug("set before changes = %s" % self.value)
        new_set = set()
        for item in self.value:
            if hasattr(item, "ref"):
                self.logger.debug("Item <<<%s>>> is a ref, trying to replace with actual object <<<%s>>>" % (item, item.ref))
                #new_set.remove(item)
                #debug begin
                newly_fetched_value = container.get_object(item.ref)
                new_set.add(newly_fetched_value)
                self.logger.debug("Item <<<%s>>> was removed, and newly fetched value <<<%s>>> was added." % (item, newly_fetched_value))
                #debug end
                #new_set.add(container.get_object(item.ref))
            else:
                self.logger.debug("Item <<<%s>>> is NOT a ref, trying to replace with scanned value" % get_string(item))
                #new_set.remove(item)
                #debug begin
                newly_scanned_value = self.scan_value(container, item)
                new_set.add(newly_scanned_value)
                self.logger.debug("Item <<<%s>>> was removed, and newly scanned value <<<%s>>> was added." % (item, newly_scanned_value))
                #debug end
                #new_set.add(self.scan_value(container, item))
        #self.logger.debug("Newly built set = %s" % new_set)
        #self.value = frozenset(new_set)
        new_frozen_set = frozenset(new_set)
        self.logger.debug("set after changes = %s" % new_frozen_set)
        #return self.value
        try:
            setattr(obj, self.name, new_frozen_set)
        except AttributeError:
            pass
        except TypeError:
            pass
        return new_frozen_set

class Config(object):
    """
    Config is an interface that defines how to read object definitions from an input source.
    """
    def read_object_defs(self):
        """Abstract method definition - should return an array of Object objects"""
        raise NotImplementedError()




"""
   Copyright 2006-2008 SpringSource (http://springsource.com), All Rights Reserved

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.       
"""
import Cookie
import logging
import re
import pickle
import types
from springpython.context import ApplicationContextAware
from springpython.aop import utils
from springpython.security import AccessDeniedException
from springpython.security import AuthenticationException
from springpython.security.context import SecurityContext
from springpython.security.context import SecurityContextHolder
from springpython.security.intercept import AbstractSecurityInterceptor
from springpython.security.intercept import ObjectDefinitionSource
from springpython.security.providers import UsernamePasswordAuthenticationToken

logger = logging.getLogger("springpython.security.web")

class Filter(object):
    """This is the interface definition of a filter. It must process a request/response."""

    def doNextFilter(self, environ, start_response):
        results = None
        try:
            nextFilter = environ["SPRINGPYTHON_FILTER_CHAIN"].next()
            if isinstance(nextFilter, tuple):
                func = nextFilter[0]
                args = nextFilter[1]
                results = func(args)
            else:
                results = nextFilter(environ, start_response)
        except StopIteration:
            pass
        
        # Apparently, passing back a generator trips up CherryPy and causes it to skip 
        # the filters. If a generator is detected, convert it to a standard array.
        if type(results) == types.GeneratorType:
            results = [line for line in results]
            
        return results
    
class FilterChain(object):
    """
    Collection of WSGI filters. It allows dynamic re-chaining of filters as the situation is needed.
    
    In order to link in 3rd party WSGI middleware, see MiddlewareFilter.
    """
    
    def __init__(self):
        self.chain = []

    def addFilter(self, filter):
        self.chain.append(filter)

    def getFilterChain(self):
        for filter in self.chain:
            yield filter
    
class FilterChainProxy(Filter, ApplicationContextAware):
    """
    This acts as filter, and delegates to a chain of filters. Each time a web page is called, it dynamically
    assembles a FilterChain, and then iterates over it. This is different than the conventional style of
    wrapping applications for WSGI, because each URL pattern might have a different chained combination
    of the WSGI filters.
    
    Because most middleware objects define the wrapped application using __init__, Spring provides
    the MiddlewareFilter, to help wrap any middleware object so that it can participate in a 
    FilterChain.
    """
    
    def __init__(self, filterInvocationDefinitionSource=None):
        """This class must be application-context aware in case it is instantiated inside an IoC container."""
        ApplicationContextAware.__init__(self)
        if filterInvocationDefinitionSource is None:
            self.filterInvocationDefinitionSource = []
        else:
            self.filterInvocationDefinitionSource = filterInvocationDefinitionSource
        self.logger = logging.getLogger("springpython.security.web.FilterChainProxy")
        self.application = None

    def __call__(self, environ, start_response):
        """This will route all requests/responses through the chain of filters."""
        filterChain = FilterChain()
        for urlPattern, chainOfFilters in self.filterInvocationDefinitionSource:
            if re.compile(urlPattern).match(environ["PATH_INFO"].lower()):
		self.logger.debug("We had a match of %s against %s" % (environ["PATH_INFO"], urlPattern))
                for filter in chainOfFilters:
                        try:
                            filterChain.addFilter(self.app_context.get_object(filter))
                        except AttributeError, e:
                            filterChain.addFilter(filter)
                break

        # Put the actual application on the end of the chain.
        if self.application:
            filterChain.addFilter(self.application)
        environ["SPRINGPYTHON_FILTER_CHAIN"] = filterChain.getFilterChain()
        return self.doNextFilter(environ, start_response)

class SessionStrategy(object):
    """
    This is an interface definition in defining access to session data. There may be many
    ways to implement session data. This makes the mechanism pluggable.
    """
    
    def getHttpSession(self, environ):
        raise NotImplementedError()

    def setHttpSession(self, key, value):
        raise NotImplementedError()

class HttpSessionContextIntegrationFilter(Filter):
    """
    This filter is meant to pull security context information from the HttpSession, and store it in the
    SecurityContextHolder. Then on the response, copy and SecurityContext information back into the HttpSession.
    """
    
    # Key to the SecurityContext data stored in an HttpSession dictionary.
    SPRINGPYTHON_SECURITY_CONTEXT_KEY = "SPRINGPYTHON_SECURITY_CONTEXT_KEY"
    
    # Class name used 
    context = SecurityContext
    
    def __init__(self, sessionStrategy=None):
        self.sessionStrategy = sessionStrategy
        self.logger = logging.getLogger("springpython.security.web.HttpSessionContextIntegrationFilter")

    def __call__(self, environ, start_response):
        """This filter copies SecurityContext information back and forth between the HttpSession and the SecurityContextHolder."""

        httpSession = self.sessionStrategy.getHttpSession(environ)
        contextWhenChainProceeded = None
        
        if httpSession is not None:

            contextFromSessionObject = None
            if self.SPRINGPYTHON_SECURITY_CONTEXT_KEY in httpSession:
                contextFromSessionObject = pickle.loads(httpSession[self.SPRINGPYTHON_SECURITY_CONTEXT_KEY])
            
            if contextFromSessionObject is not None:
                if isinstance(contextFromSessionObject, SecurityContext):
                    self.logger.debug("Obtained from SPRINGPYTHON_SECURITY_CONTEXT_KEY a valid SecurityContext and set "
                                        + "to SecurityContextHolder: '%s'" % contextFromSessionObject)
                    SecurityContextHolder.setContext(contextFromSessionObject)
                else:
                    self.logger.warn("SPRINGPYTHON_SECURITY_CONTEXT_KEY did not contain a SecurityContext but contained: '%s'" % contextFromSessionObject
                                        + "'; are you improperly modifying the HttpSession directly (you should always use " 
                                        + "SecurityContextHolder) or using the HttpSession attribute reserved for this class? "
                                        + "- new SecurityContext instance associated  with SecurityContextHolder")
                    SecurityContextHolder.setContext(self.generateNewContext())
            else:
                self.logger.debug("HttpSession returned null object for SPRINGPYTHON_SECURITY_CONTEXT_KEY " +
                                    "- new SecurityContext instance associated with SecurityContextHolder")
                SecurityContextHolder.setContext(self.generateNewContext())
                
        else:
            self.logger.debug("No HttpSession currently exists - new SecurityContext instance associated with SecurityContextHolder")
            SecurityContextHolder.setContext(self.generateNewContext())
            
        self.logger.debug("Setting contextWhenChainProceeded to %s" % SecurityContextHolder.getContext())
        contextWhenChainProceeded = str(SecurityContextHolder.getContext())
             
        results = self.doNextFilter(environ, start_response)

        self.sessionStrategy.setHttpSession(self.SPRINGPYTHON_SECURITY_CONTEXT_KEY,
                                            pickle.dumps(SecurityContextHolder.getContext()))
        self.logger.debug("SecurityContext stored to HttpSession: '%s'" % SecurityContextHolder.getContext())

        SecurityContextHolder.clearContext()
        self.logger.debug("SecurityContextHolder cleared out, as request processing completed")

        return results
            
    def setContext(self, clazz):
        """This is a factory setter. The context parameter is used to create new security context objects."""
        self.context = clazz
        
    def generateNewContext(self):
        """This is a factory method that instantiates the assigned class, and populates it with an empty token."""
        context = self.context()
        context.authentication = UsernamePasswordAuthenticationToken()
        return context

    def saveContext(self):
        self.sessionStrategy.setHttpSession(self.SPRINGPYTHON_SECURITY_CONTEXT_KEY,
                                            pickle.dumps(SecurityContextHolder.getContext()))

class RedirectStrategy(object):
    """
    This class provides a mechanism to redirect users to another page. Currently, it returns a 
    standard forwarding message to the browser. This may not be the most efficient, but it guarantees
    the entire WSGI stack is processed on both request and response.
    """
    
    def redirect(self, url):
        """This is a 0-second redirect."""
        return """<META HTTP-EQUIV="Refresh" CONTENT="0; URL=%s">""" % url

class AuthenticationProcessingFilter(Filter):
    """
    This filter utilizes the authentication manager to make sure the requesting person is authenticated.
    It expects the SecurityContextHolder to be populated when it runs, so it is always good to preceed it
    with the HttpSessionContextIntegrationFilter.
    """
    
    def __init__(self, auth_manager=None, alwaysReauthenticate=False):
        self.auth_manager = auth_manager
        self.alwaysReauthenticate = alwaysReauthenticate
        self.logger = logging.getLogger("springpython.security.web.AuthenticationProcessingFilter")
        
    def __call__(self, environ, start_response):
        """
        Check if the user is trying to access the login url. Then see if they are already authenticated (and
        alwaysReauthenticate is disabled). Finally, try to authenticate the user. If successful, stored credentials
        in SecurityContextHolder. Otherwise, redirect to the login page.
        """
        # If the user is already authenticated, skip this filter.
        if not self.alwaysReauthenticate and SecurityContextHolder.getContext().authentication.isAuthenticated():
            self.logger.debug("You are not required to reauthenticate everytime, and appear to already be authenticted, access GRANTED.")
            return self.doNextFilter(environ, start_response)
        
        try:
            # Authenticate existing credentials using the authentication manager.
            token = SecurityContextHolder.getContext().authentication
            self.logger.debug("Trying to authenticate %s using the authentication manager" % token)
            SecurityContextHolder.getContext().authentication = self.auth_manager.authenticate(token)
            self.logger.debug("%s was successfully authenticated, access GRANTED." % token.username)
        except AuthenticationException, e:
            self.logger.debug("Authentication failure, access DENIED.")
            raise

        return self.doNextFilter(environ, start_response)

    def logout(self):
        SecurityContextHolder.getContext().authentication = UsernamePasswordAuthenticationToken()

class FilterInvocation:
    """Holds objects associated with a WSGI filter, such as environ. This is the web-application equivalent to MethodInvocation."""
    
    def __init__(self, environ):
        self.environ = environ

    def requestUrl(self):
        return self.environ["PATH_INFO"]

class AbstractFilterInvocationDefinitionSource(ObjectDefinitionSource):
    """Abstract implementation of ObjectDefinitionSource."""
    
    def get_attributes(self, obj):
        try:
            return self.lookupAttributes(obj.requestUrl())
        except AttributeError:
            raise TypeError("obj must be a FilterInvocation")

    def lookupAttributes(self, url):
        raise NotImplementedError()

class RegExpBasedFilterInvocationDefinitionMap(AbstractFilterInvocationDefinitionSource):
    """
    Maintains a list of ObjectDefinitionSource's associated with different HTTP request URL regular expression patterns.
    
    Regular expressions are used to match a HTTP request URL against a ConfigAttributeDefinition. The order of registering
    the regular expressions is very important. The system will identify the first matching regular expression for a given
    HTTP URL. It will not proceed to evaluate later regular expressions if a match has already been found.
    
    Accordingly, the most specific regular expressions should be registered first, with the most general regular expressions registered last.
    """
    
    def __init__(self, obj_def_source):
        self.obj_def_source = obj_def_source

    def lookupAttributes(self, url):
        if self.obj_def_source:
            for rule, attr in self.obj_def_source:
                if re.compile(rule).match(url):
                    return attr 
        return None

class FilterSecurityInterceptor(Filter, AbstractSecurityInterceptor):
    """
    Performs security handling of HTTP resources via a filter implementation.

    The ObjectDefinitionSource required by this security interceptor is of type AbstractFilterInvocationDefinitionSource.

    Refer to AbstractSecurityInterceptor for details on the workflow.
    """
    
    # Key to the FilterSecurityInterceptor's token data stored in an HttpSession dictionary.
    SPRINGPYTHON_FILTER_SECURITY_INTERCEPTOR_KEY = "SPRINGPYTHON_FILTER_SECURITY_INTERCEPTOR_KEY"
    
    def __init__(self, auth_manager = None, access_decision_mgr = None, obj_def_source = None, sessionStrategy=None):
        Filter.__init__(self)
        AbstractSecurityInterceptor.__init__(self, auth_manager, access_decision_mgr, obj_def_source)
        self.sessionStrategy = sessionStrategy
        self.obj_def_source = obj_def_source

    def __setattr__(self, name, value):
        if name == "obj_def_source" and value is not None:
            self.__dict__[name] = RegExpBasedFilterInvocationDefinitionMap(value)
        else:
            self.__dict__[name] = value

    def obtain_obj_def_source(self):
        return self.obj_def_source

    def __call__(self, environ, start_response):
        httpSession = self.sessionStrategy.getHttpSession(environ)
	self.logger.debug("Trying to check if you are authorized for this.")
        fi = FilterInvocation(environ)
        token = self.before_invocation(fi)
        if httpSession is not None:
            httpSession[self.SPRINGPYTHON_FILTER_SECURITY_INTERCEPTOR_KEY] = token

        return self.doNextFilter(environ, start_response)

        if httpSession is not None and self.SPRINGPYTHON_FILTER_SECURITY_INTERCEPTOR_KEY in httpSession:
            token = httpSession[self.SPRINGPYTHON_FILTER_SECURITY_INTERCEPTOR_KEY]
            self.after_invocation(token, None)

        return results

class ExceptionTranslationFilter(Filter):
    """
    Handles any AccessDeniedException and AuthenticationException thrown within the filter chain.

    This filter is necessary because it provides the bridge between Python exceptions and HTTP responses.
    It is solely concerned with maintaining the user interface. This filter does not do any actual security enforcement.
    
    If an AuthenticationException is detected, the filter will launch the authenticationEntryPoint. This allows common
    handling of authentication failures originating from any subclass of AuthenticationProcessingFilter.
    
    If an AccessDeniedException is detected, the filter will launch the accessDeniedHandler. This allows common
    handling of access failures originating from any subclass of AbstractSecurityInterceptor.
    """
    
    def __init__(self, authenticationEntryPoint=None, accessDeniedHandler=None, redirectStrategy=None):
        Filter.__init__(self)
        self.authenticationEntryPoint = authenticationEntryPoint
        self.accessDeniedHandler = accessDeniedHandler
        self.logger = logging.getLogger("springpython.security.web.ExceptionTranslationFilter")
        
    def __call__(self, environ, start_response):
        try:
            return self.doNextFilter(environ, start_response)
        except AuthenticationException, e:
            self.logger.debug("AuthenticationException => %s, redirecting through authenticationEntryPoint" % e)
            return self.authenticationEntryPoint(environ, start_response)
        except AccessDeniedException, e:
            self.logger.debug("AccessDeniedException => %s, redirect through accessDeniedHandler" % e)
            return self.accessDeniedHandler(environ, start_response)

class AuthenticationProcessingFilterEntryPoint(Filter):
    """This object holds the location of the login form, and is used to commence a redirect to that form."""
    
    def __init__(self, loginFormUrl=None, redirectStrategy=None):
        Filter.__init__(self)
        self.loginFormUrl = loginFormUrl
        self.redirectStrategy = redirectStrategy
        self.logger = logging.getLogger("springpython.security.web.AuthenticationProcessingFilterEntryPoint")

    def __call__(self, environ, start_response):
        self.logger.debug("Redirecting to login page %s" % self.loginFormUrl)
        return self.redirectStrategy.redirect(self.loginFormUrl)
    
class AccessDeniedHandler(Filter):
    """Used by ExceptionTranslationFilter to handle an AccessDeniedException."""
    
    def __init__(self):
        Filter.__init__(self)
        
class SimpleAccessDeniedHandler(AccessDeniedHandler):
    """A simple default implementation of the AccessDeniedHandler interface."""
    
    def __init__(self, errorPage=None, redirectStrategy=None):
        AccessDeniedHandler.__init__(self)
        self.errorPage = errorPage
        self.redirectStrategy = redirectStrategy
        self.logger = logging.getLogger("springpython.security.web.SimpleAccessDeniedHandler")
        
    def __call__(self, environ, start_response):
        self.logger.debug("Redirecting to error page %s" % self.errorPage)
        return self.redirectStrategy.redirect(self.errorPage)

class MiddlewareFilter(Filter):
    """
    This filter allows you to wrap any WSGI-compatible middleware and use it as a Spring Python filter.
    This is primary because lots of middleware objects requires the wrapped WSGI app to be included
    in the __init__ method. Spring's IoC container currently doesn't support constructor arguments.
    """
    
    def __init__(self, clazz = None, appAttribute = None):
        Filter.__init__(self)
        self.clazz = clazz
        self.appAttribute = appAttribute

    def __setattr__(self, name, value):
        if name == "clazz" and value is not None:
            self.__dict__[name] = value
            self.middleware = utils.getClass(value)(None)
        else:
            self.__dict__[name] = value

    def __call__(self, environ, start_response):
        setattr(self.middleware, self.appAttribute, environ["SPRINGPYTHON_FILTER_CHAIN"].next())
        return self.middleware(environ, start_response)



"""
   Copyright 2006-2008 SpringSource (http://springsource.com), All Rights Reserved

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.       
"""

# stdlib
import locale
from string import Template
from cStringIO import StringIO

# Spring Python
from springpython.jms import JMSException
from springpython.jms import DEFAULT_DELIVERY_MODE

__all__ = ["MessageConverter", "JmsTemplate", "TextMessage"]

# These attributes have special meaning to connection factories.
reserved_attributes = set(("text", "jms_correlation_id", "jms_delivery_mode",
    "jms_destination", "jms_expiration", "jms_message_id", "jms_priority",
    "jms_redelivered", "jms_reply_to", "jms_timestamp", "max_chars_printed", 
    "JMS_IBM_Report_Exception", "JMS_IBM_Report_Expiration", "JMS_IBM_Report_COA",
    "JMS_IBM_Report_COD", "JMS_IBM_Report_PAN", "JMS_IBM_Report_NAN", 
    "JMS_IBM_Report_Pass_Msg_ID", "JMS_IBM_Report_Pass_Correl_ID",
    "JMS_IBM_Report_Discard_Msg", "JMSXGroupID", "JMSXGroupSeq", "JMS_IBM_Feedback",
    "JMS_IBM_Last_Msg_In_Group", "JMSXUserID", "JMS_IBM_PutTime", "JMS_IBM_PutDate", 
    "JMSXAppID"))

# Magic methods are also forbidden.
reserved_attributes.update(set(dir(object) + ["__weakref__", "__dict__", "__module__"]))

        
text_message_template = """
JMS message class: jms_text
  jms_delivery_mode:  $jms_delivery_mode
  jms_expiration:     $jms_expiration
  jms_priority:       $jms_priority
  jms_message_id:     $jms_message_id
  jms_timestamp:      $jms_timestamp
  jms_correlation_id: $jms_correlation_id
  jms_destination:    $jms_destination
  jms_reply_to:       $jms_reply_to
  jms_redelivered:    $jms_redelivered
""".lstrip()

class MessageConverter(object):
    def to_message(self, object_to_be_converted_to_a_message):
        raise NotImplementedError("Should be implemented by subclasses")

    def from_message(self, message_to_be_converted_to_an_object):
        raise NotImplementedError("Should be implemented by subclasses")

class JmsTemplate(object):
    def __init__(self, factory=None, delivery_persistent=None,
            priority=None, time_to_live=None, message_converter=None, 
            default_destination=None):
    
        self.factory = factory
        
        # QoS
        self.delivery_persistent = delivery_persistent
        self.priority = priority
        self.time_to_live = time_to_live
        
        self.message_converter = message_converter
        self.default_destination = default_destination
        
    def convert_and_send(self, object_, destination=None):
        if not self.message_converter:
            raise JMSException("Couldn't send the message, no message converter set")
            
        self.send(self.message_converter.to_message(object_), destination)
        
    def send(self, message, destination=None):
        if isinstance(message, basestring):
            message = TextMessage(message)
        
        if destination:
            dest = destination
        elif self.default_destination:
            dest = self.default_destination
        else:
            raise JMSException("No destination given and no default destination set")
        
        message.jms_destination = dest
        
        self.factory.send(message, dest)
        
    def receive(self, destination=None, timeout=1000):
        
        if destination:
            dest = destination
        elif self.default_destination:
            dest = self.default_destination
        else:
            raise JMSException("No destination given and no default destination set")
        
        return self.factory.receive(dest, timeout)
        
    def receive_and_convert(self, destination=None, timeout=1000):
        if not self.message_converter:
            raise JMSException("Couldn't receive a message, no message converter set")
            
        return self.message_converter.from_message(self.receive(destination, timeout))
        
    def open_dynamic_queue(self):
        return self.factory.open_dynamic_queue()
        
    def close_dynamic_queue(self, dynamic_queue_name):
        self.factory.close_dynamic_queue(dynamic_queue_name)

class TextMessage(object):
    def __init__(self, text=None, jms_correlation_id=None, jms_delivery_mode=None, 
            jms_destination=None, jms_expiration=None, jms_message_id=None, 
            jms_priority=None, jms_redelivered=None, jms_reply_to=None,
            jms_timestamp=None, max_chars_printed=100):
    
        self.text = text
        self.jms_correlation_id = jms_correlation_id
        
        self.jms_delivery_mode = jms_delivery_mode or DEFAULT_DELIVERY_MODE
        
        self.jms_destination = jms_destination
        self.jms_expiration = jms_expiration
        self.jms_message_id = jms_message_id
        self.jms_priority = jms_priority
        self.jms_redelivered = jms_redelivered
        self.jms_reply_to = jms_reply_to
        self.jms_timestamp = jms_timestamp
        self.max_chars_printed = max_chars_printed
        
    def __str__(self):
        basic_data = {
            "jms_delivery_mode": self.jms_delivery_mode,
            "jms_expiration":self.jms_expiration,
            "jms_priority":self.jms_priority,
            "jms_message_id":self.jms_message_id,
            "jms_timestamp":self.jms_timestamp,
            "jms_correlation_id":self.jms_correlation_id,
            "jms_destination":self.jms_destination,
            "jms_reply_to":self.jms_reply_to,
            "jms_redelivered":self.jms_redelivered,
        }
        
        buff = StringIO()
        buff.write(Template(text_message_template).safe_substitute(basic_data))
        
        user_attrs = set(dir(self)) - reserved_attributes
        user_attrs = list(user_attrs)
        user_attrs.sort()
        
        if user_attrs:
            for user_attr in user_attrs:
                user_attr_value = getattr(self, user_attr)
                
                if isinstance(user_attr_value, unicode):
                    user_attr_value = user_attr_value.encode("utf-8")
                    
                buff.write("  %s:%s\n" % (user_attr, user_attr_value))
                
        if self.text != None:
            text_to_show = self.text[:self.max_chars_printed]
            
            if isinstance(text_to_show, unicode):
                text_to_show = text_to_show.encode("utf-8")
                
            buff.write(text_to_show)
            
            if len(text_to_show) < len(self.text):
                omitted = locale.format("%d", (len(self.text) - len(text_to_show)), True)
                buff.write("\nAnother ")
                buff.write(omitted)
                buff.write(" character(s) omitted")
        else:
            buff.write("<None>")

        value = buff.getvalue()
        buff.close()
        
        return value



#!/usr/bin/env python
"""
   Copyright 2006-2008 SpringSource (http://springsource.com), All Rights Reserved

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
"""
import re
import sys

try:
    from distribute.core import setup
except ImportError:
    from setuptools import setup

if sys.version_info < (2, 6):
    print "Spring Python only supports Python 2.6 and higher"
    sys.exit(1)

setup(name='springpython',
      version='1.2.0.FINAL',
      description='Spring Python',
      long_description='Spring Python is an offshoot of the Java-based SpringFramework, targeted for Python. Spring provides many useful features, and I wanted those same features available when working with Python.',
      author='Greg L. Turnquist',
      author_email='greg.turnquist at springsource dot com',
      url='http://springpython.webfactional.com',
      platforms = ["Python >= 2.6"],
      license='Apache Software License (http://www.apache.org/licenses/LICENSE-2.0)',
      scripts=['plugins/coily'],
      packages=['springpython',
                'springpython.aop',
                'springpython.jms',
                'springpython.config',
                'springpython.container',
                'springpython.context',
                'springpython.database',
                'springpython.factory',
                'springpython.remoting',
                'springpython.remoting.hessian',
                'springpython.remoting.pyro',
                'springpython.security',
                'springpython.security.context',
                'springpython.security.providers',
                'springpython.security.userdetails'
                ],
      package_data={'springpython': ["README", "COPYRIGHT", "LICENSE.txt"]},
      download_url="http://s3.amazonaws.com/dist.springframework.org/release/EXTPY/springpython-1.2.0.FINAL.tar.gz",
      classifiers=["License :: OSI Approved :: Apache Software License",
                   "Intended Audience :: Developers",
                   "Development Status :: 5 - Production/Stable",
                   "Topic :: Software Development :: Libraries :: Python Modules",
                   "Programming Language :: Python",
                   "Operating System :: OS Independent"
                   ]

     )




"""
   Copyright 2006-2008 SpringSource (http://springsource.com), All Rights Reserved

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.       
"""
from springpython.security import SecurityException
from springpython.security.context import ThreadLocalSecurityContextHolderStrategy
from springpython.security.context import GlobalSecurityContextHolderStrategy

"""
This represents a static object that holds the context of the current session.
"""

# Currently supported strategies
MODE_THREADLOCAL = "THREADLOCAL"
MODE_GLOBAL = "GLOBAL"

# Local settings used to track strategy configuration
settings = {"strategyName" : MODE_GLOBAL, "strategy" : None, "initialized" : False }

def initialize():
    global settings

    if settings["strategyName"] == MODE_THREADLOCAL:
        settings["strategy"] = ThreadLocalSecurityContextHolderStrategy()
    elif settings["strategyName"] == MODE_GLOBAL:
        settings["strategy"] = GlobalSecurityContextHolderStrategy()
    else:
        raise SecurityException("We don't support strategy type %s" % settings["strategyName"])

    settings["initialized"] = True

def setStrategy(newStrategyName):
    global settings
    settings["strategyName"] = newStrategyName
    initialize()

def clearContext():
    if not settings["initialized"]:
        initialize()
    settings["strategy"].clearContext()

def getContext():
    """Retrieve the context, based on the strategy."""
    if not settings["initialized"]:
        initialize()
    return settings["strategy"].getContext()

def setContext(context):
    """Store the context, based on the strategy."""
    if not settings["initialized"]:
        initialize()
    settings["strategy"].setContext(context)



"""
   Copyright 2006-2008 SpringSource (http://springsource.com), All Rights Reserved

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.       
"""
import logging
from springpython.security import AccessDeniedException

logger = logging.getLogger("springpython.security.vote")

class AccessDecisionVoter:
    """
    Indicates a class is responsible for voting on authorization decisions.
    
    The coordination of voting (ie polling AccessDecisionVoters, tallying
    their responses, and making the final authorization decision) is performed
    by an AccessDecisionManager. 
    """
    ACCESS_ABSTAIN = 0
    ACCESS_DENIED = 1
    ACCESS_GRANTED = 2
    access_dict = { ACCESS_ABSTAIN: "ACCESS_ABSTAIN", ACCESS_DENIED: "ACCESS_DENIED", ACCESS_GRANTED: "ACCESS_GRANTED" }
    
    def supports(self, attr):
        """
        Indicates whether this AccessDecisionVoter is able to vote on the passed in object.
        """
        raise NotImplementedError()
    
    def vote(self, authentication, object, config):
        """Indicates whether or not access is granted."""
        raise NotImplementedError()

class RoleVoter(AccessDecisionVoter):
    """
    Votes if any ConfigAttribute.getAttribute() starts with a prefix indicating that it is a role.
    The default prefix string is ROLE_, but this may be overriden to any value. It may also be set
    to empty, which means that essentially any attribute will be voted on. As described further
    below, the effect of an empty prefix may not be quite desireable.
    
    Abstains from voting if no configuration attribute commences with the role prefix. Votes to
    grant access if there is an exact matching GrantedAuthority to a ConfigAttribute starting
    with the role prefix. Votes to deny access if there is no exact matching GrantedAuthority
    to a ConfigAttribute starting with the role prefix.
    
    An empty role prefix means that the voter will vote for every ConfigAttribute. When there
    are different categories of ConfigAttributes used, this will not be optimal since the voter
    will be voting for attributes which do not represent roles. However, this option may be of
    some use when using preexisting role names without a prefix, and no ability exists to prefix
    them with a role prefix on reading them in, such as provided for example in JdbcDaoImpl.
    
    All comparisons and prefixes are case sensitive.
    """
    def __init__(self, role_prefix = "ROLE_"):
        self.role_prefix = role_prefix
        self.logger = logging.getLogger("springpython.security.vote.RoleVoter")

    def supports(self, attr):
        """This voter will support a list, or a string starting with
        the same characters as the set prefix.
        """
        if isinstance(attr, list) or (attr is not None and attr.startswith(self.role_prefix)):
            return True
        else:
            return False

    def vote(self, authentication, invocation, config):
        """Grant access if any of the granted authorities matches any of the required
        roles.
        """
        results = self.ACCESS_ABSTAIN
        for attribute in config:
            if self.supports(attribute):
                self.logger.debug("This %s role voter will vote whether user has %s" % (self.role_prefix, attribute))
                results = self.ACCESS_DENIED
                for authority in authentication.granted_auths:
                    if attribute == authority:
                        self.logger.debug("This user has %s in %s. Vote for GRANTED!" % (attribute, authentication.granted_auths))
                        return self.ACCESS_GRANTED

        if results == self.ACCESS_ABSTAIN:
            self.logger.debug("This %s voter is abstaining from voting" % self.role_prefix)
        elif results == self.ACCESS_DENIED:
            self.logger.debug("This %s voter did NOT find the required credentials in %s. Vote for DENIED!" % (self.role_prefix, authentication.granted_auths))

        return results

    def __str__(self):
        return "<'%s' role voter>" % self.role_prefix

class AbstractAclVoter(AccessDecisionVoter):
    """
    May/may not need this class
    """
    pass

class LabelBasedAclVoter(AbstractAclVoter):
    """
     * This Acl voter will evaluate methods based on labels applied to incoming arguments. It will
     * only check methods that have been properly tagged in the MethodSecurityInterceptor with the
     * value stored in <b>attr_indicating_labeled_op</b>. If a method has been tagged, then
     * it examines each argument, and if the argument implements {@link LabeledData}, then it will
     * asses if the user's list of granted authorities matches.<p>
     * <p>
     * By default, if none of the arguments are labeled, then the access will be granted. This can
     * be overridden by setting <b>allow_access_if_no_attr_labeled</b> to false in the Spring
     * context file.<p>
     * <p>
     * In many situations, different values are linked together to define a common label, it is 
     * necessary to define a map in the application context that links user-assigned label access
     * to domain object labels. This is done by setting up the <b>label_dict</b> in the application
     * context.<p>
     * 
     * @author Greg Turnquist
     * @see org.acegisecurity.intercept.method.aopalliance.MethodSecurityInterceptor
    """
    def __init__(self, label_dict = None, allow_access_if_no_attr_labeled = False, attr_indicating_labeled_op = ""):
        if label_dict is None:
            self.label_dict = {}
        else:
            self.label_dict = label_dict
        self.allow_access_if_no_attr_labeled = allow_access_if_no_attr_labeled
        self.attr_indicating_labeled_op = attr_indicating_labeled_op
        self.logger = logging.getLogger("springpython.security.vote.LabelBasedAclVoter")

    def supports(self, attr):
    	if isinstance(attr, list) or (attr == self.attr_indicating_labeled_op):
            return True
        else:
            return False

    def vote(self, authentication, invocation, config):
        result = self.ACCESS_ABSTAIN;
        
        for attribute in config:
            if self.supports(attribute):
                result = self.ACCESS_DENIED;
            
                userLabels = []
            
                for label in authentication.granted_auths:
                    if label in self.label_dict:
                        userLabels.extend(self.label_dict[label])

                labeledArguments = [arg.getLabel() for arg in invocation.args if hasattr(arg, "getLabel")]
                matches = [arg for arg in labeledArguments if arg in userLabels]
                misses  = [arg for arg in labeledArguments if arg not in userLabels]
                self.logger.debug("Arguments: %s Matches: %s Misses: %s User labels: %s" % (labeledArguments, matches, misses, userLabels))

                if len(matches) > 0 and misses == []:
                    self.logger.debug("Access is granted!")
                    return self.ACCESS_GRANTED;
                elif labeledArguments == []:
                    if self.allow_access_if_no_attr_labeled:
                        self.logger.debug("Access is granted, since there are no attributes set!")
                        return self.ACCESS_GRANTED;
                    else:
                        self.logger.debug("Access is denied, since there are no attributes set!")
                        return self.ACCESS_DENIED;

            self.logger.debug("No matches, so returning %s" % self.access_dict[result])
            return result;

    def __str__(self):
        return "<'%s' label-based ACL voter>" % self.label_dict

class AccessDecisionManager:
    """"
    Makes a final access control (authorization) decision.
    """
    def __init__(self, access_decision_voters = [], allow_if_all_abstain = False):
        self.access_decision_voters = access_decision_voters
        self.allow_if_all_abstain = allow_if_all_abstain

    def decide(self, authentication, object, config):
        """Resolves an access control decision for the passed parameters."""
        raise NotImplementedError()


    def supports(self, attr):
        """
        Indicates whether the AccessDecisionManager implementation is able to
        provide access control decisions for the indicated secured object type.
        """
        raise NotImplementedError()
    
class AffirmativeBased(AccessDecisionManager):
    """
    Simple concrete implementation of AccessDecisionManager that grants access
    if any AccessDecisionVoter returns an affirmative response.
    """
    def __init__(self, access_decision_voters = [], allow_if_all_abstain = False):
        AccessDecisionManager.__init__(self, access_decision_voters, allow_if_all_abstain)
        self.logger = logging.getLogger("springpython.security.vote.AffirmativeBased")

    def decide(self, authentication, invocation, config):
        """
        This concrete implementation simply polls all configured
        AccessDecisionVoters and grants access if any AccessDecisionVoter voted affirmatively.
        """
        for voter in self.access_decision_voters:
            if voter.supports(config) and \
               voter.vote(authentication, invocation, config) == AccessDecisionVoter.ACCESS_GRANTED:
                self.logger.debug("Received affirmative vote from %s, granting access." % voter)
                return
        raise AccessDeniedException("Access is denied.")

class ConsensusBased(AccessDecisionManager):
    """
    Simple concrete implementation of AccessDecisionManager that uses a consensus-based approach.
    """
    def __init__(self, access_decision_voters = [], allow_if_all_abstain = False):
        AccessDecisionManager.__init__(self, access_decision_voters, allow_if_all_abstain)
        self.allow_if_tied = True
        self.logger = logging.getLogger("springpython.security.vote.ConsensusBased")

    def decide(self, authentication, invocation, config):
        """
        This concrete implementation simply polls all configured AccessDecisionVoters
        and upon completion determines the consensus of granted vs denied responses.
        """
        granted_votes = []
        denied_votes = []
        for voter in self.access_decision_voters:
            if voter.supports(config):
                vote = voter.vote(authentication, invocation, config)
                if vote == AccessDecisionVoter.ACCESS_GRANTED:
                    granted_votes.append(voter)
                if vote == AccessDecisionVoter.ACCESS_DENIED:
                    denied_votes.append(voter)
        if len(granted_votes) > len(denied_votes):
            self.logger.debug("%s granted votes, %s denial votes, granting access." % (len(granted_votes), len(denied_votes)))
            return
        elif len(granted_votes) < len(denied_votes):
            self.logger.debug("%s granted votes, %s denial votes, denying access." % (len(granted_votes), len(denied_votes)))
            raise AccessDeniedException("Access is denied.")
        elif self.allow_if_tied:
            self.logger.debug("%s granted votes, %s denial votes, granting access." % (len(granted_votes), len(denied_votes)))
            return
        else:
            self.logger.debug("%s granted votes, %s denial votes, denying access." % (len(granted_votes), len(denied_votes)))
            raise AccessDeniedException("Access is denied.")

class UnanimousBased(AccessDecisionManager):
    """
    Simple concrete implementation of AccessDecisionManager that requires all voters to
    abstain or grant access.
    """
    def __init__(self, access_decision_voters = [], allow_if_all_abstain = False):
        AccessDecisionManager.__init__(self, access_decision_voters, allow_if_all_abstain)
        self.logger = logging.getLogger("springpython.security.vote.UnanimousBased")

    def decide(self, authentication, invocation, config):
        """
        This concrete implementation polls all configured AccessDecisionVoters for
        each ConfigAttribute and grants access if only grant votes were received.
        """
        for voter in self.access_decision_voters:
            if voter.supports(config) and \
               voter.vote(authentication, invocation, config) == AccessDecisionVoter.ACCESS_DENIED:
                self.logger.debug("Received denial vote from %s, denying access" % voter)
                raise AccessDeniedException("Access is denied.")
